//===========================================================
//
//  Game Maker Tools - GMT
//
// - GMT 는 SDL에서 좀 더 쉽게 개발을 하기 위하여 기획된 라이브러리 입니다.
//  간단한 루틴 만으로 화면을 구성하고 제어할 수 있도록 제공하는 것이 1차적 목표가 되었습니다.
//  GMT 는 SDL과 통합되어져 사용되어지며, GMT를 이용하여 제어를 해야 합니다.
//
//===========================================================
//
// - 2009년 9월 8일 업데이트
//
//  1. Frame 오브젝트가 Display 오브젝트로 변경 되어졌습니다. 내부에 있던 라벨도 삭제 되었습니다.
//  2. List 오브젝트가 추가되었습니다. 이 오브젝트는 여러개 중 한가지를 선택할 수 있는 오브젝트 입니다.
//  3. Label 오브젝트가 추가 되었습니다. 이는 글자 출력을 위한 오브젝트 입니다.
//
//===========================================================
//
// - 2009년 9월 12일 업데이트
//
//  1. 기존의 unsigned int형 오브젝트 ID 가 내부적으로 GMT_ObjectID 형태로 변경 됩니다. 그룹이 추가되면서 변경 되어졌으나, 외부적으로는 변경 사항이 없습니다.
//  2. 시스템에 사용되는 폰트 크기를 저장할 수 있도록 변경 되었습니다. 이에 따라 폰트 사이즈를 변경 할 수도 있게 되었습니다.
//  3. Group화가 가능해 졌습니다. CreateGroup으로 그룹을 만들고, SetGroup으로 그룹에 묶을수 있습니다. 기존에 사용하던 모든 함수가 GroupID에 대응되게 변경 되었습니다.
//
//===========================================================
//
// - 2009년 9월 30일 업데이트
//
//  1. 기존 OpenGL 검사 루틴이 컴파일러 차원에서 프로그램 차원으로 변경 되었습니다.
//    -> 이제 확장이 지원되지 않으면 우회하여 함수를 호출하도록 변경 되어졌습니다.
//
//===========================================================
//
// - 2009년 10월 1일 업데이트
//
//  1. 몇몇 그래픽 카드에서 2^n 승 이외의 텍스쳐를 로드하지 못하는 문제를 수정하였습니다.
//
//===========================================================
//
// - 2009년 10월 2일 업데이트
//
//  1. 모듈 라이브러리가 추가 되었습니다.
//    -> JavaScript 처럼 각각의 오브젝트에 여러가지 상황에 따른 이벤트 모듈을 적재할 수 있습니다. 단, 현재는 모듈만 적재 됩니다.
//  2. 모듈에서 사용 가능한 새로운 함수인 GMT_MODULE_* 들이 추가 되었습니다.
//    -> GMT_MODULE_GetObject - 특정한 오브젝트를 얻어옵니다. 옵션 인자로 여러가지 옵션을 줄 수 있습니다.
//
//===========================================================
//
// - 추후 업데이트 계획
//
//  1. List 오브젝트의 내부 항목을 선택할 수 있도록 수정 됩니다.
//  4. 정렬 기능이 추가 됩니다. 내부 오브젝트간 정렬을 지원하여 원하는 순서와 위치에 오브젝트를 위치시킬 수 있게 됩니다.
//  5. 공간의 추가 - 현재는 평면 공간인데 언제든 3D 직교 공간 / 3D 투영 공간 / 2D 직교 공간을 오갈 수 있도록 변경 됩니다.
//  6. 오브젝트의 회전 추가 - 이 부분이 추가 되면서 사각형 선택 알고리즘의 변경도 불가피 해 질듯 싶습니다.
//  7. 색 반전 / 오브젝트의 좌우 반전 / 오브젝트의 상하 반전 - 각각 OpenGL 함수를 통해 자동으로 구현하도록 합니다.(상하 / 좌우 반전은 UV값 조절, 색 반전은 간단한 알고리즘)
//  8. 오브젝트 함수의 추가 - 각 오브젝트마다 개별적인 함수를 가질 수 있고, 이 함수는 Plug-in 형식으로 외부 프로세스가 되도록 할 예정입니다.
//
//===========================================================
//
//===========================================================

#ifndef __GMT__H__
#define __GMT__H__

#ifdef WIN32

#include <windows.h>

#endif

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>

#include "gmt_glext.h"

#include "gmt_objsystem.h"

// GMT 시스템 이벤트 타입. SDL 과의 연동 되어져 사용 됩니다.
#define GMT_MESSAGE   SDL_USEREVENT

// FPS 측정을 위한 숨겨진 변수들
struct __GMT_FPS_CHECK_STRUCT__
{

    unsigned int fpscounter;                                                        // fps 값을 저장하고 있을 fps 카운터 변수
    unsigned int nfps;                                                                // 측정된 fps 값
    char strfps[32];                                                                   // 측정된 값을 저장하는 문자열

};

// GMT 시스템 클레스, 이 클레스는 외부에서 사용되어지면 안됩니다. 보이지 않는 곳에서 사용 되어져야 합니다.
class GMTSYSTEM
{
private:

    __GMT_FPS_CHECK_STRUCT__ FPS;                                                // FPS 측정을 위한 구조체

    SDL_Surface *mainscreen;                                                           // 메인 화면을 저장할 메인 스크린 변수

    GMTOBJECTSYSTEM ObjSystem;                                                  // 오브젝트 시스템을 가집니다.

public:
    // 생성자와 소멸자, 큰 의미는 없다.
    GMTSYSTEM();
    ~GMTSYSTEM();

    // 초기화 루틴
    void InitSystem(void);
    void QuitSystem(void);

    // 화면 생성 루틴, Windows 에서는 CreateWindow 함수가 존재하므로 이름을 바꾼다.
#ifdef WIN32
    void CreateWindows
#else
    void CreateWindow
#endif
    (unsigned short int w, unsigned short int h, unsigned char bpp);

    // 화면 크기 조정 루틴
    void ResizeWindow(unsigned short int w, unsigned short int h);

    // OpenGL 화면의 크기를 조정 한다.
    void Resize(GLsizei w, GLsizei h);

    // 화면을 지우는 루틴
    void ClearScreen( void );

    // 화면을 재 갱신하는 루틴
    void Flip(void);

    // 화면의 가로, 세로 크기를 구하는 루틴
    unsigned short int GetScreenWidth(void);
    unsigned short int GetScreenHeight(void);

    // 오브젝트를 등록하는 루틴
    unsigned int CreateObject( unsigned int nObjType, unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h );

    // 그룹을 등록하는 루틴
    unsigned int CreateGroup( void );

    // 오브젝트와 마우스간의 검사
    void CheckObject( unsigned short int x, unsigned short int y );

    // 오브젝트와 마우스 클릭을 검사
    void CheckClick( unsigned short int x, unsigned short int y, unsigned char btn, unsigned char state );

    // 모듈을 적재하는 함수
    void SetModule(unsigned int nID, unsigned int nEventType, MODULE Module);

    // 그룹을 형성하는 함수
    void SetGroup(unsigned int gID, unsigned int nID);

    // 부모 자식간의 관계를 설정 하는 함수
    void SetParent(unsigned int pID, unsigned int cID);

    // 오브젝트의 속성을 제어하는 루틴
    void SetLabel(unsigned int nID, const char *Label);

    // 버튼 오브젝트의 색상을 제어하는 루틴
    void SetButtonColor(unsigned int nID, GMT_Color under, GMT_Color over, GMT_Color click);

    // 오브젝트의 BG 색을 설정 하는 함수.
    void SetBGColor(unsigned int nID, GMT_Color bgcolor);

    // 오브젝트의 배경 이미지를 설정하는 함수
    void SetBGImg(unsigned int nID, const char *path);

    // over, under, click 과 같은 상황 값이 나눠져 있는 오브젝트에 대해 각 상황별 이미지를 넣는다.
    void SetButtonImg(unsigned int nID, const char *under, const char *over, const char *click);

    // 오브젝트의 외부 선 굵기를 결정 한다.
    void SetLineWidth(unsigned int nID, unsigned char LineWidth);

    // 버튼의 각 상황별 선 굵기를 결정 한다.
    void SetButtonLineWidth(unsigned int nID, unsigned char underWidth, unsigned char overWidth, unsigned char clickWidth);

    // 오브젝트를 보이게 하거나 투명하게 한다.
    void SetVisibleObj(unsigned int nID, unsigned char IsVisible);

    // 오브젝트를 보이도록 설정 한다.
    void VisibleObject(unsigned int nID);

    // 오브젝트를 보이지 않도록 설정 한다.
    void UnvisibleObject(unsigned int nID);

    // 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
    void SetObjAlpha(unsigned int nID, unsigned char Value);

    // 오브젝트에서 알파 렌더링을 수행 할지를 설정
    void SetAbleAlpha(unsigned int nID, unsigned char IsAlpha);

    // 버튼 오브젝트에서 알파 값을 설정하여 투명도를 조절 한다.
    void SetButtonAlpha(unsigned int nID, unsigned char uValue, unsigned char oValue, unsigned char cValue);

    // 오브젝트를 삭제하는 함수
    void DeleteObject(unsigned int nID);

    // 오브젝트의 라벨 정렬을 설정하는 함수
    void SetLabelAlign(unsigned int nID, unsigned char hAlign, unsigned char vAlign);

    // 라벨의 문자열을 얼마나 띄울지 설정하는 함수
    void SetLabelOffset(unsigned int nID, short int offsetX, short int offsetY);

    // 하위 항목을 가지는 오브젝트에 대해 하위 항목을 추가하는 함수
    void AddItem(unsigned int nID, const char *String);

    // 폰트의 크기를 변경하는 함수
    void ChangeFontSize(unsigned char fontsize);

    // 폰트의 종류를 변경하는 함수
    void ChangeFont(const char *path);

    // 임시
    const char *GetFps(void);

    // 모듈내 외에서 사용된다. 특정 오브젝트를 구해온다.
    void MODULE_GetObject(unsigned char nReturnType, unsigned char nOption, void* Data);

};

// 외부에서는 사용되어지지 않는 GMT_Perror 루틴, 에러 원인을 출력 합니다.
void GMT_Perror(const char *String);

// GMT 초기화 루틴
int GMT_Init(void);

// GMT 종료 루틴
void GMT_Quit(void);

// GMT 화면 생성 루틴
void GMT_CreateWindow(unsigned short int w, unsigned short int h, unsigned char bpp);

// GMT 이벤트 제어 루틴
int GMT_GetEvent(SDL_Event *event);

// GMT 화면을 지울 색상을 저장 한다.
void GMT_ClearColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a);

// GMT 화면을 원하는 색으로 지운다.
void GMT_ClearScreen(void);

// GMT 화면을 재 갱신한다.
void GMT_Flip(void);

// 오브젝트를 생성하고 화면에 추가 한다.
unsigned int GMT_CreateLabel(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h);
unsigned int GMT_CreateList(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h);
unsigned int GMT_CreateButton(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h);
unsigned int GMT_CreateDisplay(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h);

// 오브젝트에 부모 자식 관계를 할당 한다.
void GMT_SetParent(unsigned int ParentID, unsigned int ChildID);
void GMT_SetChild(unsigned int ChildID, unsigned int ParentID);

// GMT 사각 형과 마우스 값 체크 함수
bool GMT_CheckMouseRect( unsigned short int x, unsigned short int y, GMT_Rect Rect );

// GMT 윈도우의 캡션을 변경 합니다.
void GMT_SetCaption(const char *Caption);

// GMT 오브젝트 속성을 제어한다.
// 라벨이 있는 오브젝트에 라벨 값을 넣는다.
void GMT_SetLabel(unsigned int nID, const char *Label);

// over, under, click 과 같은 상황 값이 나눠져 있는 오브젝트에 대해 각 상황별 색상을 넣는다.
void GMT_SetButtonColor(unsigned int nID, GMT_Color under, GMT_Color over, GMT_Color click);

// 오브젝트의 BG 색을 설정 하는 함수.
void GMT_SetBGColor(unsigned int nID, GMT_Color bgcolor);

// 오브젝트의 배경 이미지를 설정하는 함수
void GMT_SetBGImg(unsigned int nID, const char *path);

// over, under, click 과 같은 상황 값이 나눠져 있는 오브젝트에 대해 각 상황별 이미지를 넣는다.
void GMT_SetButtonImg(unsigned int nID, const char *under, const char *over, const char *click);

// 오브젝트의 외부 선 굵기를 결정 한다.
void GMT_SetLineWidth(unsigned int nID, unsigned char LineWidth);

// 버튼의 각 상황별 선 굵기를 결정 한다.
void GMT_SetButtonLineWidth(unsigned int nID, unsigned char underWidth, unsigned char overWidth, unsigned char clickWidth);

// 오브젝트를 보이게 하거나 투명하게 한다.
void GMT_SetVisibleObj(unsigned int nID, unsigned char IsVisible);

// 오브젝트를 보이도록 설정 한다.
void GMT_VisibleObject(unsigned int nID);

// 오브젝트를 보이지 않도록 설정 한다.
void GMT_UnvisibleObject(unsigned int nID);

// 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
void GMT_SetObjAlpha(unsigned int nID, unsigned char Value);

// 오브젝트에서 알파 렌더링을 수행 할지를 설정
void GMT_SetAbleAlpha(unsigned int nID, unsigned char IsAlpha);

// 버튼 오브젝트에서 알파 값을 설정하여 투명도를 조절 한다.
void GMT_SetButtonAlpha(unsigned int nID, unsigned char uValue, unsigned char oValue, unsigned char cValue);

// 특정한 오브젝트를 삭제하는 함수
void GMT_DeleteObject(unsigned int nID);

// 라벨의 정렬 옵션을 변경하는 함수
void GMT_SetLabelAlign(unsigned int nID, unsigned char nHorAlign, unsigned char nVerAlign);

// 라벨의 문자열을 얼마나 띄울지 설정하는 함수
void GMT_SetLabelOffset(unsigned int nID, short int offsetX, short int offsetY);

// GMT_Event 시리즈, 이벤트 구조체에서 오브젝트의 ID를 반환 합니다.
unsigned int GMT_EVENT_GetObjID( SDL_Event *event);

// GMT_Event 시리즈, 이벤트 구조체에서 오브젝트의 타입을 반환 합니다.
unsigned char GMT_EVENT_GetObjType( SDL_Event *event );

// GMT_AddItem 으로 오브젝트에 해당 아이템을 넣는다.
void GMT_AddItem(unsigned int nID, const char *String);

// 폰트의 크기를 변경하는 함수
void GMT_ChangeFontSize(unsigned char fontsize);

// 폰트의 종류를 변경하는 함수
void GMT_ChangeFont(const char *path);

// 그룹을 만드는 함수
unsigned int GMT_CreateGroup(void);

// 그룹에 속하게 하는 함수
void GMT_SetGroup(unsigned int gID, unsigned int nID);

// 프레임 측정은 임시 입니다.
// 타이머를 이용하여 타이틀에 프레임을 표시하는 함수.
void GMT_ShowFrame(void);

// 새롭게 추가된 모듈 적재 함수
void GMT_SetModule(unsigned int nID, unsigned int nEventType, MODULE Module);

// 새롭게 추가된 모듈 내에서 사용하게 될 함수
void GMT_MODULE_GetObject(unsigned char nReturnType, unsigned char nOption, void* Data);

// 타이머 함수
unsigned int TimerFunc(Uint32 interval, void *param);

#endif
