#ifndef __GMT_OBJSYSTEM__H__
#define __GMT_OBJSYSTEM__H__

// 어쩔수 없이..... SDL_ttf는 이곳에서 사용.
#include "SDL/SDL_ttf.h"

#include "gmt_linked.h"

// GMT Object System 클레스, 이 클레스는 GMT Object들을 관리 감독 합니다.
class GMTOBJECTSYSTEM
{

private:
    // 임시, 폰트 를 저장해 둔다.
    TTF_Font *font;
    char FontPath[512];                                                             // 폰트를 저장해 둔다.
    unsigned char FontSize;                                                       // 폰트의 크기를 저장하는 변수

    GMTOBJECT node[GMT_MAX_OBJECT];                                       // 오브젝트 수 만큼의 노드
    GMTGROUP gnode[GMT_MAX_GROUP];                                       // 총 그룹 수 만큼의 노드

    // 현재 수행된 오브젝트의 ID를 임시로 저장해 둔다.
    unsigned int nProcID;

    // 비어있는 오브젝트 링크드 리스트
    GMTOBJLINKEDLIST EmptyList;

    // 처리해야 되는 오브젝트 링크드 리스트
    GMTOBJLINKEDLIST ProcList;

    // 비어 있는 그룹 링크드 리스트
    GMTGROUPLINKEDLIST EmptyGroupList;

    // 추가된 그룹 링크드 리스트
    GMTGROUPLINKEDLIST GroupList;

public:
    // 시스템 초기화 함수
    void InitSystem(void);

    // 오브젝트를 등록하는 함수
    unsigned int InsertObject( unsigned int nObjType, unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h );

    // 그룹을 등록하는 함수
    unsigned int InsertGroup( void );

    // 오브젝트 리스트를 처리하는 함수
    void ProcListPorcss(void);

    // 마우스와 오브젝트 간의 관계 처리 함수
    void ProcMotionProcess(unsigned short int x, unsigned short int y);
    void ProcClickProcess(unsigned short int x, unsigned short int y, unsigned char btn, unsigned char state);

    // 그룹을 설정하는 함수
    void SetGroup(unsigned int gID, unsigned int nID);

    // 부모 자식을 설정 합니다.
    void SetParent(unsigned int pID, unsigned int cID);

    // 특정한 ID를 가진 리스트를 구해 냅니다.
    GMTOBJECT* GetObject(unsigned int nID);

    // 특정한 ID를 가진 그룹을 구해 냅니다.
    GMTGROUP* GetGroup(unsigned int gID);

    // 특정한 ID를 가진 오브젝트를 Pop 해 옵니다.
    GMTOBJECT* PopObject(unsigned int nID);

    // 오브젝트에 모듈을 적재하는 함수
    void SetModule(unsigned int nID, unsigned int nEventType, MODULE Module);

    // 특정한 ID의 오브젝트에 라벨을 넣습니다.
    void SetLabel(unsigned int nID, const char *Label);

    // 특정한 ID의 오브젝트에 각 상황별 색상을 넣습니다.
    void SetButtonColor(unsigned int nID, GMT_Color under, GMT_Color over, GMT_Color click);

    // 특정한 ID의 오브젝트에 배경 색상을 넣습니다.
    void SetBGColor(unsigned int nID, GMT_Color bgcolor);

    // 특정한 ID의 오브젝트에 배경 이미지를 넣습니다.
    void SetBGImg(unsigned int nID, GMT_Image Image, void *pixels);

    // 특정한 ID의 오브젝트에 각 상황별 이미지를 넣습니다.
    void SetButtonImg(unsigned int nID, GMT_Image uimg, void *upixels, GMT_Image oimg, void *opixels, GMT_Image cimg, void *cpixels);

    // 오브젝트의 외부 선 굵기를 결정 한다.
    void SetLineWidth(unsigned int nID, unsigned char LineWidth);

    // 버튼의 각 상황별 선 굵기를 결정 한다.
    void SetButtonLineWidth(unsigned int nID, unsigned char underWidth, unsigned char overWidth, unsigned char clickWidth);

    // 오브젝트를 보이게 하거나 투명하게 한다.
    void SetVisibleObj(unsigned int nID, unsigned char IsVisible);

    // 오브젝트를 보이도록 설정 한다.
    void VisibleObject(unsigned int nID);

    // 오브젝트를 보이지 않도록 설정 한다.
    void UnvisibleObject(unsigned int nID);

    // 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
    void SetObjAlpha(unsigned int nID, unsigned char Value);

    // 오브젝트에서 알파 렌더링을 수행 할지를 설정
    void SetAbleAlpha(unsigned int nID, unsigned char IsAlpha);

    // 버튼 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
    void SetButtonAlpha(unsigned int nID, unsigned char uValue, unsigned char oValue, unsigned char cValue);

    // 특정 오브젝트를 삭제 하는 함수
    void DeleteObject(unsigned int nID);

    // 특정 오브젝트의 라벨 정렬을 결정하는 함수
    void SetLabelAlign(unsigned int nID, unsigned char hAlign, unsigned char vAlign);

    // 라벨의 문자열을 얼마나 띄울지 설정하는 함수
    void SetLabelOffset(unsigned int nID, short int offsetX, short int offsetY);

    // 하위 항목을 가지는 오브젝트에 대해 하위 항목을 추가하는 함수
    void AddItem(unsigned int nID, const char *String);

    // 폰트의 크기를 변경하는 함수
    void ChangeFontSize(unsigned char fontsize);

    // 폰트의 종류를 변경하는 함수
    void ChangeFont(const char *path);

    // 오브젝트를 반환하는 함수
    void MODULE_GetObject(unsigned char nReturnType, unsigned char nOption, void *Data);

};

#endif
