#include "./../header/gmt.h"

// GMT에서 사용되는 glExtension 들은 void* 형태로 정의되어져 있다.
PFNGLBLENDCOLOREXTPROC glBlendColorEXT;
PFNGLWINDOWPOS2IARBPROC glWindowPos2iARB;

GMTSYSTEM *GMTSystem;

//==========================================================================================
//
// GMTSYSTEM 클레스 부분
//
//==========================================================================================

// 화면 생성 루틴, Windows일 경우와 아닐 경우에 컴파일 방법이 달라진다.
void GMTSYSTEM::
#ifndef WIN32
    CreateWindow
#else
    CreateWindows
#endif
(unsigned short int w, unsigned short int h, unsigned char bpp)
{

    // SDL 비디오를 생성 한다.
    if(!(mainscreen = SDL_SetVideoMode(w, h, bpp, SDL_HWSURFACE | SDL_OPENGLBLIT | SDL_RESIZABLE)))
    {

        GMT_Perror("윈도우 창 생성에 실패 했습니다.");

    }

    // OpenGL 비디오를 재 생성 한다.
    Resize(w, h);

}


// GMTSYSTEM 클레스 화면 지우는 루틴
void GMTSYSTEM::ClearScreen(void)
{

    glClear(GL_COLOR_BUFFER_BIT);

}

// GMTSYSTEM 클레스 화면 재 갱신 루틴
void GMTSYSTEM::Flip(void)
{

    ObjSystem.ProcListPorcss();

    FPS.fpscounter++;

    SDL_GL_SwapBuffers();

}

void GMTSYSTEM::ResizeWindow(unsigned short int w, unsigned short int h)
{

    // SDL 비디오를 재 조정 한다.
    mainscreen = SDL_SetVideoMode(w, h, mainscreen->format->BitsPerPixel, mainscreen->flags);

    // OpenGL 비디오를 재 생성 한다.
    Resize(w, h);

}

// OpenGL의 비디오 크기를 재 갱신 합니다.
void GMTSYSTEM::Resize(GLsizei w, GLsizei h)
{

    glViewport( 0, 0, w, h );

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluOrtho2D(0, w, 0, h);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

}

// 버튼을 등록 하는 함수
unsigned int GMTSYSTEM::CreateObject( unsigned int nObjType, unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h )
{

    return ObjSystem.InsertObject(nObjType, x, y, w, h);

}

// 그룹을 등록 하는 함수
unsigned int GMTSYSTEM::CreateGroup( void )
{

    return ObjSystem.InsertGroup();

}

// 그룹을 형성하는 함수
void GMTSYSTEM::SetGroup(unsigned int gID, unsigned int nID)
{

    ObjSystem.SetGroup(gID, nID);

}

// 모듈을 적재하는 함수
void GMTSYSTEM::SetModule(unsigned int nID, unsigned int nEventType, MODULE Module)
{

    ObjSystem.SetModule(nID, nEventType, Module);

}

// 마우스와 오브젝트의 처리
void GMTSYSTEM::CheckObject( unsigned short int x, unsigned short int y )
{

    ObjSystem.ProcMotionProcess(x, y);

}

// 마우스와 오브젝트의 클릭 처리
void GMTSYSTEM::CheckClick( unsigned short int x, unsigned short int y, unsigned char btn, unsigned char state)
{

    ObjSystem.ProcClickProcess(x, y, btn, state);

}

// 부모와 자식의 관계를 설정
void GMTSYSTEM::SetParent(unsigned int pID, unsigned int cID)
{

    ObjSystem.SetParent(pID, cID);

}

// 오브젝트의 속성을 변경 하는 함수
void GMTSYSTEM::SetLabel(unsigned int nID, const char *Label)
{

    ObjSystem.SetLabel(nID, Label);

}

void GMTSYSTEM::SetButtonColor(unsigned int nID, GMT_Color under, GMT_Color over, GMT_Color click)
{

    ObjSystem.SetButtonColor(nID, under, over, click);

}

// 오브젝트의 BG 색을 설정 하는 함수.
void GMTSYSTEM::SetBGColor(unsigned int nID, GMT_Color bgcolor)
{

    ObjSystem.SetBGColor(nID, bgcolor);

}

// 오브젝트의 배경 이미지를 설정하는 함수
void GMTSYSTEM::SetBGImg(unsigned int nID, const char *path)
{

    // 먼저 이미지를 로드 합니다.
    SDL_Surface *temp;
    GMT_Image tempimg;

    temp = IMG_Load(path);

    tempimg.imgWidth = temp->w;
    tempimg.imgHeight = temp->h;
    tempimg.bpp = temp->format->BitsPerPixel;

    // 로드한 이미지 데이터를 넘깁니다.
    ObjSystem.SetBGImg(nID, tempimg, temp->pixels);

}

// over, under, click 과 같은 상황 값이 나눠져 있는 오브젝트에 대해 각 상황별 이미지를 넣는다.
void GMTSYSTEM::SetButtonImg(unsigned int nID, const char *under, const char *over, const char *click)
{

    // 먼저 이미지를 로드 합니다.
    SDL_Surface *untemp, *ovtemp, *cltemp;
    GMT_Image unimg, ovimg, climg;

    untemp = IMG_Load(under);
    ovtemp = IMG_Load(over);
    cltemp = IMG_Load(click);

    unimg.imgWidth = untemp->w;
    unimg.imgHeight = untemp->h;
    unimg.bpp = untemp->format->BitsPerPixel;

    ovimg.imgWidth = ovtemp->w;
    ovimg.imgHeight = ovtemp->h;
    ovimg.bpp = ovtemp->format->BitsPerPixel;

    climg.imgWidth = cltemp->w;
    climg.imgHeight = cltemp->h;
    climg.bpp = cltemp->format->BitsPerPixel;

    // 로드한 이미지 데이터를 넘깁니다.
    ObjSystem.SetButtonImg(nID, unimg, untemp->pixels, ovimg, ovtemp->pixels, climg, cltemp->pixels );

}

// 오브젝트의 외부 선 굵기를 결정 한다.
void GMTSYSTEM::SetLineWidth(unsigned int nID, unsigned char LineWidth)
{

    ObjSystem.SetLineWidth(nID, LineWidth);

}

// 버튼의 각 상황별 선 굵기를 결정 한다.
void GMTSYSTEM::SetButtonLineWidth(unsigned int nID, unsigned char underWidth, unsigned char overWidth, unsigned char clickWidth)
{

    ObjSystem.SetButtonLineWidth(nID, underWidth, overWidth, clickWidth);

}

// 오브젝트를 보이게 하거나 투명하게 한다.
void GMTSYSTEM::SetVisibleObj(unsigned int nID, unsigned char IsVisible)
{

    ObjSystem.SetVisibleObj(nID, IsVisible);

}

// 오브젝트를 보이도록 설정 한다.
void GMTSYSTEM::VisibleObject(unsigned int nID)
{

    ObjSystem.VisibleObject(nID);

}

// 오브젝트를 보이지 않도록 설정 한다.
void GMTSYSTEM::UnvisibleObject(unsigned int nID)
{

    ObjSystem.UnvisibleObject(nID);

}

// 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
void GMTSYSTEM::SetObjAlpha(unsigned int nID, unsigned char Value)
{

    ObjSystem.SetObjAlpha(nID, Value);

}

// 오브젝트에서 알파 렌더링을 수행 할지를 설정
void GMTSYSTEM::SetAbleAlpha(unsigned int nID, unsigned char IsAlpha)
{

    ObjSystem.SetAbleAlpha(nID, IsAlpha);

}

// 버튼 오브젝트의 상황별 알파 값을 설정
void GMTSYSTEM::SetButtonAlpha(unsigned int nID, unsigned char uValue, unsigned char oValue, unsigned char cValue)
{

    ObjSystem.SetButtonAlpha(nID, uValue, oValue, cValue);

}

// 특정 오브젝트를 삭제하는 루틴
void GMTSYSTEM::DeleteObject(unsigned int nID)
{

    ObjSystem.DeleteObject(nID);

}

// 라벨의 정렬을 설정하는 루틴
void GMTSYSTEM::SetLabelAlign(unsigned int nID, unsigned char hAlign, unsigned char vAlign)
{

    ObjSystem.SetLabelAlign(nID, hAlign, vAlign);

}

// 라벨의 문자열을 얼마나 띄울지 설정하는 함수
void GMTSYSTEM::SetLabelOffset(unsigned int nID, short int offsetX, short int offsetY)
{

    ObjSystem.SetLabelOffset(nID, offsetX, offsetY);

}

// 하위 항목을 가지는 오브젝트에 대해 하위 항목을 추가하는 함수
void GMTSYSTEM::AddItem(unsigned int nID, const char *String)
{

    ObjSystem.AddItem(nID, String);

}

// 폰트의 크기를 변경하는 함수
void GMTSYSTEM::ChangeFontSize(unsigned char fontsize)
{

    ObjSystem.ChangeFontSize(fontsize);

}

// 폰트의 종류를 변경하는 함수
void GMTSYSTEM::ChangeFont(const char *path)
{

    ObjSystem.ChangeFont(path);

}

// 모듈내 외에서 사용된다. 특정 오브젝트를 구해온다.
void GMTSYSTEM::MODULE_GetObject(unsigned char nReturnType, unsigned char nOption, void* Data)
{

    // 임시로 nReturnType을 이용한다. 추후에 변경 됨.
    ObjSystem.MODULE_GetObject(nReturnType, nOption, Data);

}

//==========================================================================================
//
// GMTOBJECTSYSTEM 클레스 부분
//
//==========================================================================================

// 오브젝트 등록 하는 함수
unsigned int GMTOBJECTSYSTEM::InsertObject( unsigned int nObjType, unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h )
{

    // 비어 있는 오브젝트 하나를 pop하고,
    GMTOBJECT *node = EmptyList.PopData();

    // 오브젝트 속성을 다 넣는다.
    node->ObjectType = nObjType;

    node->IsVisible = true;
    node->IsAble = true;

    node->Position.x = x;
    node->Position.y = y;
    node->Position.w = w;
    node->Position.h = h;

    // 오브젝트 특성에 맞는 오브젝트 타입을 동적 할당한다.
    switch( nObjType)
    {

        case GMT_OBJ_LABEL:

            node->ObjAttribute = (GMTLABEL*)malloc(sizeof(GMTLABEL));
            memset(node->ObjAttribute, 0, sizeof(GMTLABEL));

        break;

        case GMT_OBJ_BUTTON:

            node->ObjAttribute = (GMTBUTTON*)malloc(sizeof(GMTBUTTON));
            memset(node->ObjAttribute, 0, sizeof(GMTBUTTON));

        break;

        case GMT_OBJ_DISPLAY:

            node->ObjAttribute = (GMTDISPLAY*)malloc(sizeof(GMTDISPLAY));
            memset(node->ObjAttribute, 0, sizeof(GMTDISPLAY));

        break;

        case GMT_OBJ_LIST:

            node->ObjAttribute = (GMTLIST*)malloc(sizeof(GMTLIST));
            memset(node->ObjAttribute, 0, sizeof(GMTLIST));

        break;

    }

    return ProcList.InsertData(node);

}

// 그룹을 생성하는 함수
unsigned int GMTOBJECTSYSTEM::InsertGroup( void )
{

    // 비어 있는 오브젝트 하나를 pop하고
    GMTGROUP *node = EmptyGroupList.PopData();

    // GroupList에 등록 한다.
    return GroupList.InsertData(node);

}

// 그룹을 설정하는 함수
void GMTOBJECTSYSTEM::SetGroup(unsigned int gID, unsigned int nID)
{

    // 먼저 해당 ID를 가진 오브젝트를 찾는다.
    GMTGROUP *group = GetGroup(gID);

    // 임시
    if(!group)printf("오브젝트가 없는데!?\n");

    // 그룹이 가진 총 ID를 1 증가 시키고, 마지막에 해당 ID를 추가한다.
    if(!group->nTotalMember++)
        group->Member = (unsigned int*)malloc(sizeof(unsigned int));
    else
        group->Member = (unsigned int*)realloc(group->Member, sizeof(unsigned int) * group->nTotalMember);

    group->Member[group->nTotalMember - 1] = nID;

}

// 부모 / 자식 관계를 설정하는 함수
void GMTOBJECTSYSTEM::SetParent(unsigned int pID, unsigned int cID)
{

    // 부모 노드를 구하고, 자식 노드는 해당 위치에서 Pop 한다.
    GMTOBJECT *parent, *child, *temp;

    parent = GetObject(pID);
    child = PopObject(cID);
    temp = parent->child;

    // 임시
    if(!parent || !child)printf("오브젝트가 없는데....?\n");

    // 부모와 자식 관계를 만든다. 부모는 자식으로 이동 가능하고, 자식은 부모의 데이터를 참조할 수 있다.
    child->parent = parent;

    // 자식의 이전과 다음은 무조건 부모이다.
    child->prev = parent;
    child->next = parent;

    // 부모에게 이미 자식이 있다면, 자식의 링크드 리스트에 추가한다.
    if(!parent->child)
        parent->child = child;
    else
    {

        // 만약 자식이 있다면 이전 자식 링크드 리스트의 끝을 찾는다.
        while( temp->next != parent )
            temp = temp->next;

        // 기존 자식의 끝의 다음은 이번 자식이다.
        temp->next = child;
        child->prev = temp;

    }

}

// 모듈을 적재하는 루틴
void GMTOBJECTSYSTEM::SetModule(unsigned int nID, unsigned int nEventType, MODULE Module)
{

    if( nID & GMT_IS_GROUP32 )
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch(nEventType)
            {

                case GMT_MODULE_ONCLICK:

                    // 모듈이 적재 되었음을 설정
                    node->IsModule = true;
                    node->EventList.OnClick = Module;

                break;

                case GMT_MODULE_ONOVER:

                    // 모듈이 적재 되었음을 설정
                    node->IsModule = true;
                    node->EventList.OnOver = Module;

                break;

                case GMT_MODULE_ONOUT:

                    // 모듈이 적재 되었음을 설정
                    node->IsModule = true;
                    node->EventList.OnOut = Module;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject( nID );

        switch(nEventType)
        {

            case GMT_MODULE_ONCLICK:

                // 모듈이 적재 되었음을 설정
                node->IsModule = true;
                node->EventList.OnClick = Module;

            break;

            case GMT_MODULE_ONOVER:

                // 모듈이 적재 되었음을 설정
                node->IsModule = true;
                node->EventList.OnOver = Module;

            break;

            case GMT_MODULE_ONOUT:

                // 모듈이 적재 되었음을 설정
                node->IsModule = true;
                node->EventList.OnOut = Module;

            break;

        }

    }

}

// 오브젝트에 라벨을 설정 하는 루틴
void GMTOBJECTSYSTEM::SetLabel(unsigned int nID, const char *Label)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_LABEL:

                    ((GMTLABEL*)node->ObjAttribute)->nHorzAlign = GMT_CENTER;
                    ((GMTLABEL*)node->ObjAttribute)->nVertAlign = GMT_MIDDLE;

                    strcpy( ((GMTLABEL*)node->ObjAttribute)->Label, Label );

                break;

                case GMT_OBJ_DISPLAY:

                    ((GMTDISPLAY*)node->ObjAttribute)->Label.nHorzAlign = GMT_CENTER;
                    ((GMTDISPLAY*)node->ObjAttribute)->Label.nVertAlign = GMT_MIDDLE;

                    strcpy( ((GMTDISPLAY*)node->ObjAttribute)->Label.Label, Label );

                break;

                case GMT_OBJ_BUTTON:

                    ((GMTBUTTON*)node->ObjAttribute)->Label.nHorzAlign = GMT_CENTER;
                    ((GMTBUTTON*)node->ObjAttribute)->Label.nVertAlign = GMT_MIDDLE;

                    strcpy( ((GMTBUTTON*)node->ObjAttribute)->Label.Label, Label );

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {

            case GMT_OBJ_LABEL:

                ((GMTLABEL*)node->ObjAttribute)->nHorzAlign = GMT_CENTER;
                ((GMTLABEL*)node->ObjAttribute)->nVertAlign = GMT_MIDDLE;

                strcpy( ((GMTLABEL*)node->ObjAttribute)->Label, Label );

            break;

            case GMT_OBJ_DISPLAY:

                ((GMTDISPLAY*)node->ObjAttribute)->Label.nHorzAlign = GMT_CENTER;
                ((GMTDISPLAY*)node->ObjAttribute)->Label.nVertAlign = GMT_MIDDLE;

                strcpy( ((GMTDISPLAY*)node->ObjAttribute)->Label.Label, Label );

            break;

            case GMT_OBJ_BUTTON:

                ((GMTBUTTON*)node->ObjAttribute)->Label.nHorzAlign = GMT_CENTER;
                ((GMTBUTTON*)node->ObjAttribute)->Label.nVertAlign = GMT_MIDDLE;

                strcpy( ((GMTBUTTON*)node->ObjAttribute)->Label.Label, Label );

            break;

        }

    }

}

// 오브젝트에 상황별 색상을 설정 하는 루틴
void GMTOBJECTSYSTEM::SetButtonColor(unsigned int nID, GMT_Color under, GMT_Color over, GMT_Color click)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_BUTTON:

                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgColor = under;
                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgColor = over;
                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgColor = click;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {
            case GMT_OBJ_BUTTON:

                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgColor = under;
                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgColor = over;
                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgColor = click;

            break;

        }

    }

}

// 특정한 ID의 오브젝트에 배경 색상을 설정 하는 루틴
void GMTOBJECTSYSTEM::SetBGColor(unsigned int nID, GMT_Color bgcolor)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {
                case GMT_OBJ_DISPLAY:

                    ((GMTDISPLAY*)node->ObjAttribute)->BgColor = bgcolor;

                break;

                case GMT_OBJ_LABEL:

                    ((GMTLABEL*)node->ObjAttribute)->LabelBGColor = bgcolor;

                break;

                case GMT_OBJ_LIST:

                    ((GMTLIST*)node->ObjAttribute)->BGColor = bgcolor;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {
            case GMT_OBJ_DISPLAY:

                ((GMTDISPLAY*)node->ObjAttribute)->BgColor = bgcolor;

            break;

            case GMT_OBJ_LABEL:

                ((GMTLABEL*)node->ObjAttribute)->LabelBGColor = bgcolor;

            break;

            case GMT_OBJ_LIST:

                ((GMTLIST*)node->ObjAttribute)->BGColor = bgcolor;

            break;

        }

    }

}

// 특정한 ID의 오브젝트에 배경 이미지를 설정 하는 루틴
void GMTOBJECTSYSTEM::SetBGImg(unsigned int nID, GMT_Image Image, void *pixels)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_DISPLAY:

                    // OpenGL 커맨드로 텍스쳐를 생성한다.
                    glGenTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);
                    glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);

                    switch( Image.bpp >> 3 )
                    {

                        case 3:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Image.imgWidth, Image.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, pixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // ¸¸¾à ÅØ½ºÃÄ »ý¼º¿¡ ½ÇÆÐ ÇÑ´Ù¸é, Æí¹ýÀ¸·Î ¸¸µì´Ï´Ù.
                                int n = 0;

                                if( Image.imgWidth > Image.imgHeight )
                                    for(n = 1 ; (Image.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (Image.imgHeight >> n) ; n++);

                                n -= 1;

                                // ±âÁ¸ ÅØ½ºÃÄ´Â ºÎ¼ü´Ï´Ù.
                                glDeleteTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);

                                // »õ·Î ÅØ½ºÃÄ¸¦ ¸¸µé°í,
                                glGenTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);

                                // ÀûÀýÇÑ Å©±âÀÇ ÅØ½ºÃÄ¸¦ »ý¼ºÇÑ´Ù.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                                // ÅØ½ºÃÄ¸¦ ¸¸µé¾î ³Ö½À´Ï´Ù.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, Image.imgWidth, Image.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, pixels);

                                // ±âÁ¸ 1.0À» ÇÏ´ø°Í¿¡ Å©±â°¡ ¹Ù²î¾úÀ¸´Ï, ÇØ´ç Å©±âµµ Á¶Á¤ÇØ ÁÝ´Ï´Ù.
                                // (Çö Å©±â / ÀüÃ¼ Å©±â) °¡ U°ª.
                                node->IsNewUV = true;

                                node->uValue = ((float)Image.imgWidth / (float)(2 << n));
                                node->vValue = ((float)Image.imgHeight / (float)(2 << n));

                            }

                        break;

                        case 4:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Image.imgWidth, Image.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, pixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // ¸¸¾à ÅØ½ºÃÄ »ý¼º¿¡ ½ÇÆÐ ÇÑ´Ù¸é, Æí¹ýÀ¸·Î ¸¸µì´Ï´Ù.
                                int n = 0;

                                if( Image.imgWidth > Image.imgHeight )
                                    for(n = 1 ; (Image.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (Image.imgHeight >> n) ; n++);

                                n -= 1;

                                // ±âÁ¸ ÅØ½ºÃÄ´Â ºÎ¼ü´Ï´Ù.
                                glDeleteTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);

                                // »õ·Î ÅØ½ºÃÄ¸¦ ¸¸µé°í,
                                glGenTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);

                                // ÀûÀýÇÑ Å©±âÀÇ ÅØ½ºÃÄ¸¦ »ý¼ºÇÑ´Ù.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                                // ÅØ½ºÃÄ¸¦ ¸¸µé¾î ³Ö½À´Ï´Ù.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, Image.imgWidth, Image.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, pixels);

                                // ±âÁ¸ 1.0À» ÇÏ´ø°Í¿¡ Å©±â°¡ ¹Ù²î¾úÀ¸´Ï, ÇØ´ç Å©±âµµ Á¶Á¤ÇØ ÁÝ´Ï´Ù.
                                // (Çö Å©±â / ÀüÃ¼ Å©±â) °¡ U°ª.
                                node->IsNewUV = true;

                                node->uValue = ((float)Image.imgWidth / (float)(2 << n));
                                node->vValue = ((float)Image.imgHeight / (float)(2 << n));

                            }

                        break;

                    }

                    break;

            }

            // 오브젝트에 이미지 출력을 활성화 합니다.
            node->ImgDraw = 1;

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {

            case GMT_OBJ_DISPLAY:

                // OpenGL 커맨드로 텍스쳐를 생성한다.
                glGenTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);
                glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);

                switch( Image.bpp >> 3 )
                {

                    case 3:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Image.imgWidth, Image.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, pixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( Image.imgWidth > Image.imgHeight )
                                for(n = 1 ; (Image.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (Image.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, Image.imgWidth, Image.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, pixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)Image.imgWidth / (float)(2 << n));
                            node->vValue = ((float)Image.imgHeight / (float)(2 << n));

                        }

                    break;

                    case 4:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Image.imgWidth, Image.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, pixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( Image.imgWidth > Image.imgHeight )
                                for(n = 1 ; (Image.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (Image.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTDISPLAY*)node->ObjAttribute)->BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, Image.imgWidth, Image.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, pixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)Image.imgWidth / (float)(2 << n));
                            node->vValue = ((float)Image.imgHeight / (float)(2 << n));

                        }

                    break;

                }

                break;

        }

        // 오브젝트에 이미지 출력을 활성화 합니다.
        node->ImgDraw = 1;

    }

}

// 특정한 ID의 오브젝트에 각 상황별 이미지를 설정 하는 루틴
void GMTOBJECTSYSTEM::SetButtonImg(unsigned int nID, GMT_Image uimg, void *upixels, GMT_Image oimg, void *opixels, GMT_Image cimg, void *cpixels)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {
                case GMT_OBJ_BUTTON:

                    // OpenGL 커맨드로 텍스쳐를 생성한다.
                    glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);
                    glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);
                    glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                    glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                    switch( uimg.bpp >> 3 )
                    {

                        case 3:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, uimg.imgWidth, uimg.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, upixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // n 승을 구하기 위한 임의의 변수
                                int n = 0;

                                if( uimg.imgWidth > uimg.imgHeight )
                                    for(n = 1 ; (uimg.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (uimg.imgHeight >> n) ; n++);

                                n -= 1;

                                // 기존의 텍스쳐는 삭제한다.
                                glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                                // 새롭게 텍스쳐를 할당한다.
                                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                                 // 빈 텍스쳐를 하나 만든다.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                                // 텍스쳐 이미지를 복사한다.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, uimg.imgWidth, uimg.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, upixels);

                                // 새롭게 UV를 검사할 루틴을 추가한다.
                                node->IsNewUV = true;

                                node->uValue = ((float)uimg.imgWidth / (float)(2 << n));
                                node->vValue = ((float)uimg.imgHeight / (float)(2 << n));

                            }

                        break;

                        case 4:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, uimg.imgWidth, uimg.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, upixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // n 승을 구하기 위한 임의의 변수
                                int n = 0;

                                if( uimg.imgWidth > uimg.imgHeight )
                                    for(n = 1 ; (uimg.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (uimg.imgHeight >> n) ; n++);

                                n -= 1;

                                // 기존의 텍스쳐는 삭제한다.
                                glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                                // 새롭게 텍스쳐를 할당한다.
                                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                                // 빈 텍스쳐를 하나 만든다.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                                // 텍스쳐 이미지를 복사한다.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, uimg.imgWidth, uimg.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, upixels);

                                // 새롭게 UV를 검사할 루틴을 추가한다.
                                node->IsNewUV = true;

                                node->uValue = ((float)uimg.imgWidth / (float)(2 << n));
                                node->vValue = ((float)uimg.imgHeight / (float)(2 << n));

                            }

                        break;

                    }

                    glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                    switch( uimg.bpp >> 3 )
                    {

                        case 3:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, oimg.imgWidth, oimg.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, opixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // n 승을 구하기 위한 임의의 변수
                                int n = 0;

                                if( oimg.imgWidth > oimg.imgHeight )
                                    for(n = 1 ; (oimg.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (oimg.imgHeight >> n) ; n++);

                                n -= 1;

                                // 기존의 텍스쳐는 삭제한다.
                                glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                                // 새롭게 텍스쳐를 할당한다.
                                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                                // 빈 텍스쳐를 하나 만든다.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                                // 텍스쳐 이미지를 복사한다.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, oimg.imgWidth, oimg.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, opixels);

                                // 새롭게 UV를 검사할 루틴을 추가한다.
                                node->IsNewUV = true;

                                node->uValue = ((float)oimg.imgWidth / (float)(2 << n));
                                node->vValue = ((float)oimg.imgHeight / (float)(2 << n));

                            }

                        break;

                        case 4:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, oimg.imgWidth, oimg.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, opixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // n 승을 구하기 위한 임의의 변수
                                int n = 0;

                                if( oimg.imgWidth > oimg.imgHeight )
                                    for(n = 1 ; (oimg.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (oimg.imgHeight >> n) ; n++);

                                n -= 1;

                                // 기존의 텍스쳐는 삭제한다.
                                glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                                // 새롭게 텍스쳐를 할당한다.
                                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                                // 빈 텍스쳐를 하나 만든다.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                                // 텍스쳐 이미지를 복사한다.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, oimg.imgWidth, oimg.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, opixels);

                                // 새롭게 UV를 검사할 루틴을 추가한다.
                                node->IsNewUV = true;

                                node->uValue = ((float)oimg.imgWidth / (float)(2 << n));
                                node->vValue = ((float)oimg.imgHeight / (float)(2 << n));

                            }

                        break;

                    }

                    glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                    switch( uimg.bpp >> 3 )
                    {

                        case 3:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, cimg.imgWidth, cimg.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, cpixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // n 승을 구하기 위한 임의의 변수
                                int n = 0;

                                if( cimg.imgWidth > cimg.imgHeight )
                                    for(n = 1 ; (cimg.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (cimg.imgHeight >> n) ; n++);

                                n -= 1;

                                // 기존의 텍스쳐는 삭제한다.
                                glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                                // 새롭게 텍스쳐를 할당한다.
                                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                                // 빈 텍스쳐를 하나 만든다.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                                // 텍스쳐 이미지를 복사한다.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, cimg.imgWidth, cimg.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, cpixels);

                                // 새롭게 UV를 검사할 루틴을 추가한다.
                                node->IsNewUV = true;

                                node->uValue = ((float)cimg.imgWidth / (float)(2 << n));
                                node->vValue = ((float)cimg.imgHeight / (float)(2 << n));

                            }

                        break;

                        case 4:

                            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, cimg.imgWidth, cimg.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, cpixels);

                            if( glGetError() == GL_INVALID_VALUE )
                           {

                                // n 승을 구하기 위한 임의의 변수
                                int n = 0;

                                if( cimg.imgWidth > cimg.imgHeight )
                                    for(n = 1 ; (cimg.imgWidth >> n) ; n++);
                                else
                                    for(n = 1 ; (cimg.imgHeight >> n) ; n++);

                                n -= 1;

                                // 기존의 텍스쳐는 삭제한다.
                                glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                                // 새롭게 텍스쳐를 할당한다.
                                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);
                                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                                // 빈 텍스쳐를 하나 만든다.
                                glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                                // 텍스쳐 이미지를 복사한다.
                                glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, cimg.imgWidth, cimg.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, cpixels);

                                // 새롭게 UV를 검사할 루틴을 추가한다.
                                node->IsNewUV = true;

                                node->uValue = ((float)cimg.imgWidth / (float)(2 << n));
                                node->vValue = ((float)cimg.imgHeight / (float)(2 << n));

                            }

                        break;

                    }

                    break;

            }

            node->ImgDraw = 1;

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {
            case GMT_OBJ_BUTTON:

                // OpenGL 커맨드로 텍스쳐를 생성한다.
                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);
                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);
                glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                switch( uimg.bpp >> 3 )
                {

                    case 3:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, uimg.imgWidth, uimg.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, upixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( cimg.imgWidth > cimg.imgHeight )
                                for(n = 1 ; (cimg.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (cimg.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, cimg.imgWidth, cimg.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, cpixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)cimg.imgWidth / (float)(2 << n));
                            node->vValue = ((float)cimg.imgHeight / (float)(2 << n));

                        }

                    break;

                    case 4:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, uimg.imgWidth, uimg.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, upixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( uimg.imgWidth > uimg.imgHeight )
                                for(n = 1 ; (uimg.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (uimg.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, uimg.imgWidth, uimg.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, upixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)uimg.imgWidth / (float)(2 << n));
                            node->vValue = ((float)uimg.imgHeight / (float)(2 << n));

                        }

                    break;

                }

                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                switch( uimg.bpp >> 3 )
                {

                    case 3:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, oimg.imgWidth, oimg.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, opixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( oimg.imgWidth > oimg.imgHeight )
                                for(n = 1 ; (oimg.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (oimg.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, oimg.imgWidth, oimg.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, opixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)oimg.imgWidth / (float)(2 << n));
                            node->vValue = ((float)oimg.imgHeight / (float)(2 << n));

                        }

                    break;

                    case 4:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, oimg.imgWidth, oimg.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, opixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( oimg.imgWidth > oimg.imgHeight )
                                for(n = 1 ; (oimg.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (oimg.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, oimg.imgWidth, oimg.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, opixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)oimg.imgWidth / (float)(2 << n));
                            node->vValue = ((float)oimg.imgHeight / (float)(2 << n));

                        }

                    break;

                }

                glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                switch( uimg.bpp >> 3 )
                {

                    case 3:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, cimg.imgWidth, cimg.imgHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, cpixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( cimg.imgWidth > cimg.imgHeight )
                                for(n = 1 ; (cimg.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (cimg.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 2 << n, 2 << n, 0, GL_RGB, GL_UNSIGNED_BYTE, 0 );

				   // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, cimg.imgWidth, cimg.imgHeight, GL_RGB, GL_UNSIGNED_BYTE, cpixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)cimg.imgWidth / (float)(2 << n));
                            node->vValue = ((float)cimg.imgHeight / (float)(2 << n));

                        }

                    break;

                    case 4:

                        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, cimg.imgWidth, cimg.imgHeight, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, cpixels);

                        if( glGetError() == GL_INVALID_VALUE )
                       {

                            // n 승을 구하기 위한 임의의 변수
                            int n = 0;

                            if( cimg.imgWidth > cimg.imgHeight )
                                for(n = 1 ; (cimg.imgWidth >> n) ; n++);
                            else
                                for(n = 1 ; (cimg.imgHeight >> n) ; n++);

                            n -= 1;

                            // 기존의 텍스쳐는 삭제한다.
                            glDeleteTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                            // 새롭게 텍스쳐를 할당한다.
                            glGenTextures(1, &((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);
                            glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].BgID);

                            // 빈 텍스쳐를 하나 만든다.
                            glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 2 << n, 2 << n, 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, 0 );

                            // 텍스쳐 이미지를 복사한다.
                            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, cimg.imgWidth, cimg.imgHeight, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, cpixels);

                            // 새롭게 UV를 검사할 루틴을 추가한다.
                            node->IsNewUV = true;

                            node->uValue = ((float)cimg.imgWidth / (float)(2 << n));
                            node->vValue = ((float)cimg.imgHeight / (float)(2 << n));

                        }


                    break;

                }

                break;

        }

        node->ImgDraw = 1;

    }

}

// 오브젝트의 외부 선 굵기를 결정 한다.
void GMTOBJECTSYSTEM::SetLineWidth(unsigned int nID, unsigned char LineWidth)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_DISPLAY:

                    ((GMTDISPLAY*)node->ObjAttribute)->LineWidth = LineWidth;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {

            case GMT_OBJ_DISPLAY:

                ((GMTDISPLAY*)node->ObjAttribute)->LineWidth = LineWidth;

            break;

        }

    }

}

// 버튼의 각 상황별 선 굵기를 결정 한다.
void GMTOBJECTSYSTEM::SetButtonLineWidth(unsigned int nID, unsigned char underWidth, unsigned char overWidth, unsigned char clickWidth)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_BUTTON:

                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].LineWidth = underWidth;
                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].LineWidth = overWidth;
                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].LineWidth = clickWidth;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {

            case GMT_OBJ_BUTTON:

                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_UNDER].LineWidth = underWidth;
                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_OVER].LineWidth = overWidth;
                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[GMT_OBJ_CLICK].LineWidth = clickWidth;

            break;

        }

    }

}

// 오브젝트를 보이게 하거나 투명하게 한다.
void GMTOBJECTSYSTEM::SetVisibleObj(unsigned int nID, unsigned char IsVisible)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            node->IsVisible = IsVisible;

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        node->IsVisible = IsVisible;

    }

}

// 오브젝트를 보이도록 설정 한다.
void GMTOBJECTSYSTEM::VisibleObject(unsigned int nID)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            node->IsVisible = GMT_VISIBLE;

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        node->IsVisible = GMT_VISIBLE;

    }

}

// 오브젝트를 보이지 않도록 설정 한다.
void GMTOBJECTSYSTEM::UnvisibleObject(unsigned int nID)
{

     // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            node->IsVisible = GMT_UNVISIBLE;

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        node->IsVisible = GMT_UNVISIBLE;

    }

}

// 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
void GMTOBJECTSYSTEM::SetObjAlpha(unsigned int nID, unsigned char Value)
{

     // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_DISPLAY:

                    ((GMTDISPLAY*)node->ObjAttribute)->AlphaValue = Value;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {

            case GMT_OBJ_DISPLAY:

                ((GMTDISPLAY*)node->ObjAttribute)->AlphaValue = Value;

            break;

        }

    }

}

// 오브젝트의 버튼의 알파 값을 설정하여 투명도를 조절 한다.
void GMTOBJECTSYSTEM::SetButtonAlpha(unsigned int nID, unsigned char uValue, unsigned char oValue, unsigned char cValue)
{

     // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );

            switch( node->ObjectType )
            {

                case GMT_OBJ_BUTTON:

                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ GMT_OBJ_UNDER ].AlphaValue = uValue;
                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ GMT_OBJ_OVER ].AlphaValue = oValue;
                    ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ GMT_OBJ_CLICK ].AlphaValue = cValue;

                break;

            }

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        switch( node->ObjectType )
        {

            case GMT_OBJ_BUTTON:

                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ GMT_OBJ_UNDER ].AlphaValue = uValue;
                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ GMT_OBJ_OVER ].AlphaValue = oValue;
                ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ GMT_OBJ_CLICK ].AlphaValue = cValue;

            break;

        }

    }

}

// 오브젝트에서 알파 렌더링을 수행 할지를 설정
void GMTOBJECTSYSTEM::SetAbleAlpha(unsigned int nID, unsigned char IsAlpha)
{

    // ID값이 그룹인지 아닌지 먼저 판별한다.
    if(nID & GMT_IS_GROUP32)
    {

        // ID 값이 그룹이라면, 그룹을 구해오고 그룹에 있는 각 멤버들을 처리 한다.
        GMTGROUP *group = GetGroup(nID);

        for(unsigned int i = 0 ; i < group->nTotalMember ; i++)
        {

            GMTOBJECT *node = GetObject( group->Member[i] );
            node->AlphaDraw = IsAlpha;

        }

    }
    else
    {

        GMTOBJECT *node = GetObject(nID);

        node->AlphaDraw = IsAlpha;

    }

}

void GMTOBJECTSYSTEM::DeleteObject(unsigned int nID)
{

    // Proc 링크드 리스트를 돌며 마우스와의 위치를 파악한다.
    for(GMTOBJECT *node = ProcList.GetRootData() ; node != ProcList.GetTailData() ; )
    {

        if( node->nID == nID )
        {

            // 오브젝트가 자식을 가지고 있는지 확인한다.
            if( node->child )
            {

                // 자식을 가지고 있다면, 자식들을 모두 Push 한다.
                GMTOBJECT *childnode = node->child;
                GMTOBJECT *childnext;

                while( node->next != childnode->next )
                {

                    childnext = childnode->next;

                    // 현재 자식노드를 ProcList에 Push 시킨다.
                    ProcList.PushData(childnode);

                    childnode = childnext;

                }

            }

            if( node->parent )
            {

                // 만약 자식 노드라면, 다음과 같은 루틴을 사용한다.
                if( node->prev == node->parent )
                {

                    // 만약 자식 노드 이면서, 마지막 자식 노드라면
                    if( node->next == node->parent )
                    {

                        node->parent->child = NULL;

                        EmptyList.PushData( node );

                        return;

                    }

                    node->parent->child = node->next;

                }
                else
                {

                    node->prev->next = node->next;

                }

                if( node->next != node->parent )
                    node->next->prev = node->prev;

                return;

            }

            // 자료를 먼저 삭제 한다.
            node->prev->next = node->next;
            node->next->prev = node->prev;

            // 현재 노드 데이터를 초기화 해 준다.
            node->child = node->parent = node->prev = node->next = NULL;

            // 자료를 Empty 에 넣는다.
            EmptyList.PushData( node );

            return;

        }

        if( node->child )
        {

            node = node->child;

        }
        else
        {

            if( node->next == node->parent )
                node = node->parent->next;
            else
                node = node->next;

        }

    }

}

// 라벨의 정렬을 설정하는 함수
void GMTOBJECTSYSTEM::SetLabelAlign(unsigned int nID, unsigned char hAlign, unsigned char vAlign)
{

    GMTOBJECT *node = GetObject(nID);

    switch( node->ObjectType )
    {

        case GMT_OBJ_LABEL:

            ((GMTLABEL*)node->ObjAttribute)->nHorzAlign = hAlign;
            ((GMTLABEL*)node->ObjAttribute)->nVertAlign = vAlign;

        break;

        case GMT_OBJ_DISPLAY:

            ((GMTDISPLAY*)node->ObjAttribute)->Label.nHorzAlign = hAlign;
            ((GMTDISPLAY*)node->ObjAttribute)->Label.nVertAlign = vAlign;

        break;

        case GMT_OBJ_BUTTON:

            ((GMTBUTTON*)node->ObjAttribute)->Label.nHorzAlign = hAlign;
            ((GMTBUTTON*)node->ObjAttribute)->Label.nVertAlign = vAlign;

        break;

        case GMT_OBJ_LIST:
        break;

    }

}

// 라벨의 문자열을 얼마나 띄울지 설정하는 함수
void GMTOBJECTSYSTEM::SetLabelOffset(unsigned int nID, short int offsetX, short int offsetY)
{

    GMTOBJECT *node = GetObject(nID);

    switch( node->ObjectType )
    {

        case GMT_OBJ_LABEL:

            ((GMTLABEL*)node->ObjAttribute)->offsetx = offsetX;
            ((GMTLABEL*)node->ObjAttribute)->offsety = offsetY;

        break;

        case GMT_OBJ_DISPLAY:

            ((GMTDISPLAY*)node->ObjAttribute)->Label.offsetx = offsetX;
            ((GMTDISPLAY*)node->ObjAttribute)->Label.offsety = offsetY;

        break;

        case GMT_OBJ_BUTTON:

            ((GMTBUTTON*)node->ObjAttribute)->Label.offsetx = offsetX;
            ((GMTBUTTON*)node->ObjAttribute)->Label.offsety = offsetY;

        break;

        case GMT_OBJ_LIST:
        break;

    }

}

void GMTOBJECTSYSTEM::AddItem(unsigned int nID, const char *String)
{

    GMTOBJECT *node = GetObject(nID);

    switch( node->ObjectType )
    {

        case GMT_OBJ_LABEL:
        break;

        case GMT_OBJ_DISPLAY:
        break;

        case GMT_OBJ_BUTTON:
        break;

        case GMT_OBJ_LIST:

            if(!((GMTLIST*)node->ObjAttribute)->nTotalItem++)
                ((GMTLIST*)node->ObjAttribute)->ListItem = (GMTLISTITEM*)malloc(sizeof(GMTLISTITEM) * ((GMTLIST*)node->ObjAttribute)->nTotalItem);
            else
                ((GMTLIST*)node->ObjAttribute)->ListItem = (GMTLISTITEM*)realloc(((GMTLIST*)node->ObjAttribute)->ListItem, sizeof(GMTLISTITEM) * ((GMTLIST*)node->ObjAttribute)->nTotalItem);

            strcpy( ((GMTLIST*)node->ObjAttribute)->ListItem[((GMTLIST*)node->ObjAttribute)->nTotalItem - 1].ItemName, String );

        break;

    }

}

// 폰트의 크기를 변경하는 함수
void GMTOBJECTSYSTEM::ChangeFontSize(unsigned char fontsize)
{

    FontSize = fontsize;

    if(!(font = TTF_OpenFont("./Data/fonts/Eunjin.ttf", FontSize)))
        GMT_Perror("font 로드에 실패 했습니다.");

}

// 폰트의 종류를 변경하는 함수
void GMTOBJECTSYSTEM::ChangeFont(const char *path)
{

    if(!(font = TTF_OpenFont(path, FontSize)))
        GMT_Perror("font 로드에 실패 했습니다.");

}

void GMTOBJECTSYSTEM::MODULE_GetObject(unsigned char nReturnType, unsigned char nOption, void *Data)
{

    GMTOBJECT *node;

    switch( nReturnType )
    {

        case GMT_OBJ_ID:

            switch( nOption )
            {

                case GMT_MYSELF:

                    // 현재 선택된 오브젝트가 선택 됩니다.
                    *((unsigned int*)Data) = nProcID;

                break;

                case GMT_OTHER:

                    // 아무거나 하나 구해냅니다.
                    node = GetObject(nProcID);

                    // 노드의 다음이나 이전의 ID를 반환 합니다.
                    if(node->next != ProcList.GetTailData())
                    {

                        *((unsigned int*)Data) = node->next->nID;
                        break;

                    }

                    if(node->prev != ProcList.GetRootData())
                    {

                        *((unsigned int*)Data) = node->prev->nID;
                        break;

                    }

                    Data = NULL;

                break;

            }

        break;

        case GMT_OBJ_OBJECT:

            switch( nOption )
            {

                case GMT_MYSELF:

                    // 현재 선택된 오브젝트가 선택 됩니다.
                    node = GetObject(nProcID);
                    *((GMTOBJECT*)Data) = *node;

                break;

                case GMT_OTHER:

                    // 아무거나 하나 구해냅니다.
                    node = GetObject(nProcID);

                    // 노드의 다음이나 이전의 ID를 반환 합니다.
                    if(node->next != ProcList.GetTailData())
                    {

                        *((GMTOBJECT*)Data) = *node->next;
                        break;

                    }

                    if(node->prev != ProcList.GetRootData())
                    {

                        *((GMTOBJECT*)Data) = *node->prev;
                        break;

                    }

                    Data = NULL;

                break;

            }


        break;

        case GMT_OBJ_TYPE:

            switch( nOption )
            {

                case GMT_MYSELF:

                    // 현재 선택된 오브젝트가 선택 됩니다.
                    node = GetObject(nProcID);
                    *((unsigned char*)Data) = node->ObjectType;

                break;

                case GMT_OTHER:

                    // 아무거나 하나 구해냅니다.
                    node = GetObject(nProcID);

                    // 노드의 다음이나 이전의 ID를 반환 합니다.
                    if(node->next != ProcList.GetTailData())
                    {

                        *((unsigned char*)Data) = node->next->ObjectType;
                        break;

                    }

                    if(node->prev != ProcList.GetRootData())
                    {

                        *((unsigned char*)Data) = node->prev->ObjectType;
                        break;

                    }

                    Data = NULL;

                break;

            }


        break;

    }

}

//==========================================================================================
//
// GMT 관련 함수 들
//
//==========================================================================================

// GMT 에러 관련 루틴
void GMT_Perror(const char *String)
{

    printf("%s\n", String);
    getchar();
    exit(1);

}

// GMT 로 라벨을 생성 하는 루틴
unsigned int GMT_CreateLabel(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h)
{

    return GMTSystem->CreateObject( GMT_OBJ_LABEL, x, y, w, h );

}

// GMT 로 리스트를 생성 하는 루틴
unsigned int GMT_CreateList(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h)
{

    return GMTSystem->CreateObject( GMT_OBJ_LIST, x, y, w, h );

}

// GMT 로 버튼을 생성 하는 루틴
unsigned int GMT_CreateButton(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h)
{

    return GMTSystem->CreateObject( GMT_OBJ_BUTTON, x, y, w, h );

}

// GMT 로 프레임을 생성 하는 루틴
unsigned int GMT_CreateDisplay(unsigned short int x, unsigned short int y, unsigned short int w, unsigned short int h)
{

    return GMTSystem->CreateObject( GMT_OBJ_DISPLAY, x, y, w, h );

}

// GMT 구문, 오브젝트에 부모 - 자식 관계를 형성
void GMT_SetParent(unsigned int ParentID, unsigned int ChildID)
{

    // 클레스로 넘긴다.
    GMTSystem->SetParent(ParentID, ChildID);

}

// GMT 구문, 오브젝트에 자식 - 부모 관계를 형성
void GMT_SetChild(unsigned int ChildID, unsigned int ParentID)
{

    // 클레스로 넘긴다.
    GMTSystem->SetParent(ParentID, ChildID);

}

// GMT 구문, 윈도우의 캡션을 변경 합니다.
void GMT_SetCaption(const char *Caption)
{

    SDL_WM_SetCaption(Caption, Caption);

}

// GMT 구문, 오브젝트에 라벨을 적용 한다.
void GMT_SetLabel(unsigned int nID, const char *Label)
{

    GMTSystem->SetLabel(nID, Label);

}

// GMT 구문, 버튼의 각 상태별 색상을 적용 한다.
void GMT_SetButtonColor(unsigned int nID, GMT_Color under, GMT_Color over, GMT_Color click)
{

    GMTSystem->SetButtonColor(nID, under, over, click);

}

// 오브젝트의 BG 색을 설정 하는 함수.
void GMT_SetBGColor(unsigned int nID, GMT_Color bgcolor)
{

    GMTSystem->SetBGColor(nID, bgcolor);

}

// 오브젝트의 배경 이미지를 설정하는 함수
void GMT_SetBGImg(unsigned int nID, const char *path)
{

    GMTSystem->SetBGImg(nID, path);

}

// over, under, click 과 같은 상황 값이 나눠져 있는 오브젝트에 대해 각 상황별 이미지를 넣는다.
void GMT_SetButtonImg(unsigned int nID, const char *under, const char *over, const char *click)
{

    GMTSystem->SetButtonImg(nID, under, over, click);

}

// 오브젝트의 외부 선 굵기를 결정 한다.
void GMT_SetLineWidth(unsigned int nID, unsigned char LineWidth)
{

    GMTSystem->SetLineWidth(nID, LineWidth);

}

// 버튼의 각 상황별 선 굵기를 결정 한다.
void GMT_SetButtonLineWidth(unsigned int nID, unsigned char underWidth, unsigned char overWidth, unsigned char clickWidth)
{

    GMTSystem->SetButtonLineWidth(nID, underWidth, overWidth, clickWidth);

}

// 오브젝트를 보이게 하거나 투명하게 한다.
void GMT_SetVisibleObj(unsigned int nID, unsigned char IsVisible)
{

    GMTSystem->SetVisibleObj(nID, IsVisible);

}

// 오브젝트를 보이도록 설정 한다.
void GMT_VisibleObject(unsigned int nID)
{

    GMTSystem->VisibleObject(nID);

}

// 오브젝트를 보이지 않도록 설정 한다.
void GMT_UnvisibleObject(unsigned int nID)
{

    GMTSystem->UnvisibleObject(nID);

}

// 오브젝트의 알파 값을 설정하여 투명도를 조절 한다.
void GMT_SetObjAlpha(unsigned int nID, unsigned char Value)
{

    GMTSystem->SetObjAlpha(nID, Value);

}

// 오브젝트에서 알파 렌더링을 수행 할지를 설정
void GMT_SetAbleAlpha(unsigned int nID, unsigned char IsAlpha)
{

    GMTSystem->SetAbleAlpha(nID, IsAlpha);

}

// 버튼 오브젝트에서 알파 값을 설정하여 투명도를 조절 한다.
void GMT_SetButtonAlpha(unsigned int nID, unsigned char uValue, unsigned char oValue, unsigned char cValue)
{

    GMTSystem->SetButtonAlpha(nID, uValue, oValue, cValue);

}

// 오브젝트를 삭제 하는 명령
void GMT_DeleteObject(unsigned int nID)
{

    GMTSystem->DeleteObject(nID);

}

// 라벨의 정렬 옵션을 조절하는 명령
void GMT_SetLabelAlign(unsigned int nID, unsigned char nHorAlign, unsigned char nVerAlign)
{

    GMTSystem->SetLabelAlign(nID, nHorAlign, nVerAlign);

}

// 라벨의 문자열을 얼마나 띄울지 설정하는 함수
void GMT_SetLabelOffset(unsigned int nID, short int offsetX, short int offsetY)
{

    GMTSystem->SetLabelOffset(nID, offsetX, offsetY);

}

void GMT_AddItem(unsigned int nID, const char *String)
{

    GMTSystem->AddItem(nID, String);

}

// GMT_Event 시리즈, 오브젝트의 타입을 반환
unsigned char GMT_EVENT_GetObjType( SDL_Event *event )
{

    return ((GMTOBJECT*)event->user.data2)->ObjectType;

}

// GMT_Event 시리즈, 오브젝트의 ID를 반환
unsigned int GMT_EVENT_GetObjID( SDL_Event *event)
{

    return ((GMTOBJECT*)event->user.data2)->nID;

}

// 폰트의 크기를 변경하는 함수
void GMT_ChangeFontSize(unsigned char fontsize)
{

    GMTSystem->ChangeFontSize(fontsize);

}

// 폰트의 종류를 변경하는 함수
void GMT_ChangeFont(const char *path)
{

    // 폰트가 유효한지 검사
    FILE *fp;

    if( (fp = fopen(path, "rb")) == NULL )
        return;

    fclose(fp);

    GMTSystem->ChangeFont(path);

}

// 그룹을 만드는 함수
unsigned int GMT_CreateGroup(void)
{

    return GMTSystem->CreateGroup();

}

// 그룹에 속하게 하는 함수
void GMT_SetGroup(unsigned int gID, unsigned int nID)
{

    GMTSystem->SetGroup(gID, nID);

}

const char *GMTSYSTEM::GetFps(void)
{

    return FPS.strfps;

}

// 측정된 fps를 적용하기, 이 함수도 임시
void GMT_ShowFrame(void)
{

    SDL_WM_SetCaption(GMTSystem->GetFps(), GMTSystem->GetFps());

}

// 모듈을 적재하는 함수
void GMT_SetModule(unsigned int nID, unsigned int nEventType, MODULE Module)
{

    GMTSystem->SetModule(nID, nEventType, Module);

}

// 특정 데이터를 반환하는 함수
void GMT_MODULE_GetObject(unsigned char nReturnType, unsigned char nOption, void* Data)
{

    // 반환 데이터 형태와 옵션에 따라서 Data에 값을 달리 쓴다.
    GMTSystem->MODULE_GetObject(nReturnType, nOption, Data);

}

// FPS 측정 하기
unsigned int TimerFunc(Uint32 interval, void *param)
{

    // 엄청 단순한 FPS 측정
    ((__GMT_FPS_CHECK_STRUCT__*)param)->nfps = ((__GMT_FPS_CHECK_STRUCT__*)param)->fpscounter;
    ((__GMT_FPS_CHECK_STRUCT__*)param)->fpscounter = 0;

    sprintf(((__GMT_FPS_CHECK_STRUCT__*)param)->strfps, "%d / 60 fps", ((__GMT_FPS_CHECK_STRUCT__*)param)->nfps);

    return interval;

}
