//===========================================================
//
//  Game Maker Tools - GMT_linked.cpp
//
//  - gmt_linked.cpp 파일은 링크드 리스트 제어 루틴 들이 구현 되어져 있습니다.
//
//===========================================================

#include "./../header/gmt.h"

// 다른 곳에서 선언된 GMTSystem 클레스.
extern GMTSYSTEM *GMTSystem;

//==========================================================================================
//
// GMTOBJLINKEDLIST 클레스 부분
//
//==========================================================================================

// 데이터 등록 함수
unsigned int GMTOBJLINKEDLIST::InsertData(GMTOBJECT *node)
{

    // node에 ID를 할당, ID값 0은 널 값으로 사용되지 않는다.
    node->nID = ++nTotalNode;

    tail.prev->next = node;
    node->prev = tail.prev;
    tail.prev = node;
    node->next = &tail;

    return nTotalNode;

}

// 루트의 다음 데이터를 구해 냅니다.
GMTOBJECT* GMTOBJLINKEDLIST::GetRootData(void)
{

    return root.next;

}

// 테일의 주소를 구해 냅니다.
GMTOBJECT* GMTOBJLINKEDLIST::GetTailData(void)
{

    return &tail;

}

// 데이터를 Push 하는 함수
void GMTOBJLINKEDLIST::PushData(GMTOBJECT *node)
{

    tail.prev->next = node;
    node->prev = tail.prev;
    node->next = &tail;
    tail.prev = node;

}

// 데이터를 Pop 하는 함수
GMTOBJECT* GMTOBJLINKEDLIST::PopData(void)
{

    GMTOBJECT *node = root.next;

    root.next = root.next->next;

    return node;

}

// 모든 링크드 리스트를 저장해 둡니다.
void GMTOBJLINKEDLIST::AllMakeList(GMTOBJECT *node)
{

    for(int i = 0 ; i < GMT_MAX_OBJECT ; i++)
        InsertData(&node[i]);

}


//==========================================================================================
//
// GMTGROUPLINKEDLIST 클레스 부분
//
//==========================================================================================

// 데이터 등록 함수
unsigned int GMTGROUPLINKEDLIST::InsertData(GMTGROUP *node)
{

    // 그룹의 gID 값을 저장, 이를 위해 약간의 편법을 쓴다.
    GMT_ObjectID GroupID;
    GroupID.nID = 0;
    GroupID.nGroupID = ++nTotalNode | GMT_IS_GROUP;

    node->gID = GroupID.nID;

    tail.prev->next = node;
    node->prev = tail.prev;
    tail.prev = node;
    node->next = &tail;

    return nTotalNode | GMT_IS_GROUP32;

}

// 루트의 다음 데이터를 구해 냅니다.
GMTGROUP* GMTGROUPLINKEDLIST::GetRootData(void)
{

    return root.next;

}

// 테일의 주소를 구해 냅니다.
GMTGROUP* GMTGROUPLINKEDLIST::GetTailData(void)
{

    return &tail;

}

// 데이터를 Push 하는 함수
void GMTGROUPLINKEDLIST::PushData(GMTGROUP *node)
{

    tail.prev->next = node;
    node->prev = tail.prev;
    node->next = &tail;
    tail.prev = node;

}

// 데이터를 Pop 하는 함수
GMTGROUP* GMTGROUPLINKEDLIST::PopData(void)
{

    GMTGROUP *node = root.next;

    root.next = root.next->next;

    return node;

}

// 모든 링크드 리스트를 저장해 둡니다.
void GMTGROUPLINKEDLIST::AllMakeList(GMTGROUP *node)
{

    for(int i = 0 ; i < GMT_MAX_GROUP ; i++)
        InsertData(&node[i]);

}
