//===========================================================
//
//  Game Maker Tools - GMT_proc.cpp
//
//  - gmt_proc.cpp 파일은 처리 관련 루틴이 모여져 있습니다.
//
//===========================================================

#include "./../header/gmt.h"
// GMT에서 사용되는 glExtension 들은 void* 형태로 정의되어져 있다.
extern PFNGLBLENDCOLOREXTPROC glBlendColorEXT;
extern PFNGLWINDOWPOS2IARBPROC glWindowPos2iARB;

// 다른 곳에서 선언된 GMTSystem 클레스.
extern GMTSYSTEM *GMTSystem;

//==========================================================================================
//
// GMTOBJECTSYSTEM 클레스 부분
//
//==========================================================================================

// 프로세스 처리 루틴
void GMTOBJECTSYSTEM::ProcListPorcss(void)
{

    // 임시로 사용될 이미지 데이터
    SDL_Surface *temp = 0;
    GMT_Color tempColor;
    int PosX, PosY;

    // Proc 링크드 리스트를 돌며 이미지를 그린다.
    for(GMTOBJECT *node = ProcList.GetRootData() ; node != ProcList.GetTailData() ; )
    {

        switch(node->ObjectType)
        {

            case GMT_OBJ_LIST:

                // 먼저 영역을 그린다.
                tempColor = ((GMTLIST*)node->ObjAttribute)->BGColor;
                glColor3f( ( (GLfloat)tempColor.r / 255.0f ), ( (GLfloat)tempColor.g / 255.0f ), ( (GLfloat)tempColor.b / 255.0f ) );
                glRectf(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y, node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - (node->Position.y + node->Position.h));

                // 이제 부터는 가지고 있는 데이터들의 수에 맞게 그린다.
                for( int i = 0 ; i < ((GMTLIST*)node->ObjAttribute)->nTotalItem ; i++ )
                {

                    if( ((GMTLIST*)node->ObjAttribute)->ListItem[i].ItemName )
                    {

                        if(temp)
                            SDL_FreeSurface(temp);

                        SDL_Color Color;
                        Color.r = 0;
                        Color.g = 0;
                        Color.b = 0;

                        temp = TTF_RenderUTF8_Blended( font, ((GMTLIST*)node->ObjAttribute)->ListItem[i].ItemName, Color );

                        if( temp )
                        {

                            glEnable(GL_BLEND);
                            glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
                            glPixelZoom(1.0f, -1.0f);

                            if( !glWindowPos2iARB )
                                glRasterPos2i( node->Position.x + 5 , ( node->Position.y + 5 + ( ( i * 5 ) + i * temp->h ) ) );
                            else
                                glWindowPos2iARB( node->Position.x + 5 , GMTSystem->GetScreenHeight() - ( node->Position.y + 5 + ( ( i * 5 ) + i * temp->h ) ) );

                            glDrawPixels(temp->w, temp->h, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, temp->pixels);
                            glDisable(GL_BLEND);

                        }

                    }

                }

            break;

            case GMT_OBJ_LABEL:

                // 먼저 영역을 그린다.
                tempColor = ((GMTLABEL*)node->ObjAttribute)->LabelBGColor;
                glColor3f( ( (GLfloat)tempColor.r / 255.0f ), ( (GLfloat)tempColor.g / 255.0f ), ( (GLfloat)tempColor.b / 255.0f ) );
                glRectf(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y, node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - (node->Position.y + node->Position.h));

                // 먼저, Label 이 있는지 검사하고 있다면 이미지를 그릴 준비를 합니다.
                if( ((GMTLABEL*)node->ObjAttribute)->Label )
                {

                    if(temp)
                        SDL_FreeSurface(temp);

                    SDL_Color Color;
                    Color.r = ((GMTLABEL*)node->ObjAttribute)->LabelColor.r;
                    Color.g = ((GMTLABEL*)node->ObjAttribute)->LabelColor.g;
                    Color.b = ((GMTLABEL*)node->ObjAttribute)->LabelColor.b;

                    //temp = TTF_RenderUTF8_Solid(font, ((GMTBUTTON*)node->ObjAttribute)->Label, Color );
                    temp = TTF_RenderUTF8_Blended(font, ((GMTLABEL*)node->ObjAttribute)->Label, Color );

                }

                if( temp )
                {

                    glEnable(GL_BLEND);
                    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
                    glPixelZoom(1.0f, -1.0f);

                    // 이 부분은 정렬 옵션에 따라 정렬 위치를 바꾼다.
                    switch( ((GMTLABEL*)node->ObjAttribute)->nHorzAlign )
                    {

                        case GMT_LEFT:

                            PosX = ( node->Position.x );

                        break;

                        case GMT_CENTER:

                            PosX = ( ( ( node->Position.x << 1 ) + node->Position.w ) >> 1 ) - ( temp->w >> 1 );

                        break;

                        case GMT_RIGHT:

                            PosX = ( ( node->Position.x ) + node->Position.w ) - ( temp->w );

                        break;

                    }

                    switch( ((GMTLABEL*)node->ObjAttribute)->nVertAlign )
                    {

                        case GMT_TOP:

                            PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->Position.y ) ) ) ) );

                        break;

                        case GMT_CENTER:

                            PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->Position.y << 1 ) + node->Position.h ) >> 1 ) ) + ( temp->h >> 1 ) );

                        break;

                        case GMT_BOTTOM:

                            PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->Position.y ) + node->Position.h ) ) ) + ( temp->h ) );

                        break;

                    }

                    if( !glWindowPos2iARB )
                        glRasterPos2i( PosX, GMTSystem->GetScreenHeight() + PosY );
                    else
                        glWindowPos2iARB( PosX , PosY );

                    glDrawPixels(temp->w, temp->h, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, temp->pixels);
                    glDisable(GL_BLEND);

                }

            break;

            case GMT_OBJ_BUTTON:

                // 먼저, Label 이 있는지 검사하고 있다면 이미지를 그릴 준비를 합니다.
                if( ((GMTBUTTON*)node->ObjAttribute)->Label.Label )
                {

                    if(temp)
                        SDL_FreeSurface(temp);

                    SDL_Color Color;
                    Color.r = ((GMTBUTTON*)node->ObjAttribute)->Label.LabelColor.r;
                    Color.g = ((GMTBUTTON*)node->ObjAttribute)->Label.LabelColor.g;
                    Color.b = ((GMTBUTTON*)node->ObjAttribute)->Label.LabelColor.b;

                    //temp = TTF_RenderUTF8_Solid(font, ((GMTBUTTON*)node->ObjAttribute)->Label, Color );
                    temp = TTF_RenderUTF8_Blended(font, ((GMTBUTTON*)node->ObjAttribute)->Label.Label, Color );

                }

                // 만약, 부모가 존재 한다면 부모의 위치 데이터를 더해 줘야 합니다.
                // 색상은 현재 오브젝트의 상태 값에 의존하여 처리 합니다.
                tempColor = ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].BgColor;
                glColor3f( ( (GLfloat)tempColor.r / 255.0f ), ( (GLfloat)tempColor.g / 255.0f ), ( (GLfloat)tempColor.b / 255.0f ) );

                if(node->parent)
                {

                    if( node->ImgDraw )
                    {

                        glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].BgID);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
                        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

                        glEnable(GL_TEXTURE_2D);
                        glEnable(GL_ALPHA_TEST);

                        glAlphaFunc(GL_GREATER, 0.9f);

                        if( node->AlphaDraw )
                        {

                            // BlendColor Extension 으로 사용
                            glEnable(GL_BLEND);

                            // glBlendColorEXT 함수가 존재 한다면, 확장을 사용한다. 없다면 우회 해서 사용한다.
                            if( !glBlendColorEXT )
                            {

                                glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

                            }
                            else
                            {

                                glBlendColorEXT(1.0f, 1.0f, 1.0f, ( ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].AlphaValue ? ((GLfloat)((GMTBUTTON*)node->ObjAttribute)->ButtonState[node->State].AlphaValue / 255.0f) : 0.0f ) );
                                glBlendFunc( GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA );

                            }

                        }

                        if(node->IsNewUV)
                        {

                            glBegin(GL_QUADS);

                                glTexCoord2f(node->uValue, 0.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                                glTexCoord2f(node->uValue, node->vValue);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                                glTexCoord2f(0.0f, node->vValue);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                                glTexCoord2f(0.0f, 0.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                            glEnd();

                        }
                        else
                        {

                            glBegin(GL_QUADS);

                                glTexCoord2f(1.0f, 0.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                                glTexCoord2f(1.0f, 1.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                                glTexCoord2f(0.0f, 1.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                                glTexCoord2f(0.0f, 0.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                            glEnd();

                        }

                        glDisable(GL_TEXTURE_2D);
                        glDisable(GL_ALPHA_TEST);

                        if( node->AlphaDraw )
                            glDisable(GL_BLEND);

                    }
                    else
                    {

                        glRectf( ( node->Position.x + node->parent->Position.x ), GMTSystem->GetScreenHeight() - ( node->parent->Position.y + node->Position.y ), ( node->Position.x + node->parent->Position.x ) + node->Position.w, GMTSystem->GetScreenHeight() - ( ( node->parent->Position.y + node->Position.y ) + node->Position.h));

                    }

                    if( ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].LineWidth )
                    {

                        glLineWidth( ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].LineWidth );
                        glColor3f(0.0f, 0.0f, 0.0f);
                        glPolygonMode(GL_FRONT, GL_LINE);

                        glBegin(GL_QUADS);

                            glVertex2i( ( node->Position.x + node->parent->Position.x ), GMTSystem->GetScreenHeight() - ( node->parent->Position.y + node->Position.y ));
                            glVertex2i( ( node->Position.x + node->parent->Position.x ), GMTSystem->GetScreenHeight() - ( ( node->parent->Position.y + node->Position.y ) + node->Position.h) );
                            glVertex2i( ( node->Position.x + node->parent->Position.x ) + node->Position.w, GMTSystem->GetScreenHeight() - ( ( node->parent->Position.y + node->Position.y ) + node->Position.h) );
                            glVertex2i( ( node->Position.x + node->parent->Position.x ) + node->Position.w, GMTSystem->GetScreenHeight() - ( node->parent->Position.y + node->Position.y ) );

                        glEnd();

                        glPolygonMode(GL_FRONT, GL_FILL);

                        glLineWidth(1.0f);

                    }

                    if( temp )
                    {

                        glEnable(GL_BLEND);
                        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
                        glPixelZoom(1.0f, -1.0f);

                        // 이 부분은 정렬 옵션에 따라 정렬 위치를 바꾼다.
                        switch( ((GMTBUTTON*)node->ObjAttribute)->Label.nHorzAlign )
                        {

                            case GMT_LEFT:

                                PosX = ( node->parent->Position.x + node->Position.x ) + ((GMTBUTTON*)node->ObjAttribute)->Label.offsetx;

                            break;

                            case GMT_CENTER:

                                PosX = ( ( ( ( node->parent->Position.x + ( node->Position.x << 1 ) ) + node->Position.w ) >> 1 ) - ( temp->w >> 1 ) )  + ((GMTBUTTON*)node->ObjAttribute)->Label.offsetx ;

                            break;

                            case GMT_RIGHT:

                                PosX = ( ( ( node->parent->Position.x + node->Position.x ) + node->Position.w ) - ( temp->w ) ) + ((GMTBUTTON*)node->ObjAttribute)->Label.offsetx;

                            break;

                        }

                        switch( ((GMTBUTTON*)node->ObjAttribute)->Label.nVertAlign )
                        {

                            case GMT_TOP:

                                PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->parent->Position.y + node->Position.y ) ) ) ) ) - ((GMTBUTTON*)node->ObjAttribute)->Label.offsety;

                            break;

                            case GMT_CENTER:

                                PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->parent->Position.y + ( node->Position.y << 1 ) ) + node->Position.h ) >> 1 ) ) + ( temp->h >> 1 ) ) - ((GMTBUTTON*)node->ObjAttribute)->Label.offsety;

                            break;

                            case GMT_BOTTOM:

                                PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->parent->Position.y + node->Position.y ) + node->Position.h ) ) ) + ( temp->h ) ) - ((GMTBUTTON*)node->ObjAttribute)->Label.offsety;

                            break;

                        }

                        if( !glWindowPos2iARB )
                            glRasterPos2i( PosX, GMTSystem->GetScreenHeight() + PosY );
                        else
                            glWindowPos2iARB( PosX , PosY );


                        glDrawPixels(temp->w, temp->h, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, temp->pixels);
                        glDisable(GL_BLEND);

                    }

                }
                else
                {

                    if( node->ImgDraw )
                    {

                        GLfloat ClearValue[4];
                        GLfloat ColorValue[4];

                        glBindTexture(GL_TEXTURE_2D, ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].BgID);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
                        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

                        glEnable(GL_TEXTURE_2D);
                        glEnable(GL_ALPHA_TEST);

                        glAlphaFunc(GL_GREATER, 0.9f);

                        if( node->AlphaDraw )
                        {

                            // BlendColor Extension 으로 사용
                            glEnable(GL_BLEND);

                            if( !glBlendColorEXT )
                            {

                                glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

                            }
                            else
                            {

                                glBlendColorEXT(1.0f, 1.0f, 1.0f, ( ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].AlphaValue ? ((GLfloat)((GMTBUTTON*)node->ObjAttribute)->ButtonState[node->State].AlphaValue / 255.0f) : 0.0f ) );
                                glBlendFunc( GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA );

                            }

                        }

                        if( node->IsNewUV )
                        {

                            glBegin(GL_QUADS);

                                glTexCoord2f(node->uValue, 0.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                                glTexCoord2f(node->uValue, node->vValue);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                                glTexCoord2f(0.0f, node->vValue);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                                glTexCoord2f(0.0f, 0.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                            glEnd();

                        }
                        else
                        {

                            glBegin(GL_QUADS);

                                glTexCoord2f(1.0f, 0.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                                glTexCoord2f(1.0f, 1.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                                glTexCoord2f(0.0f, 1.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                                glTexCoord2f(0.0f, 0.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                            glEnd();

                        }

                        glDisable(GL_TEXTURE_2D);
                        glDisable(GL_ALPHA_TEST);

                        if( node->AlphaDraw )
                            glDisable(GL_BLEND);

                    }
                    else
                    {

                        glRectf(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y, node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - (node->Position.y + node->Position.h));

                    }


                    if( ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].LineWidth )
                    {

                        glLineWidth( ((GMTBUTTON*)node->ObjAttribute)->ButtonState[ node->State ].LineWidth );
                        glColor3f(0.0f, 0.0f, 0.0f);
                        glPolygonMode(GL_FRONT, GL_LINE);

                        glBegin(GL_QUADS);

                            glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                            glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                            glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                            glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                        glEnd();

                        glPolygonMode(GL_FRONT, GL_FILL);

                        glLineWidth(1.0f);

                    }

                    if( temp )
                    {

                        glEnable(GL_BLEND);
                        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
                        glPixelZoom(1.0f, -1.0f);

                        // 이 부분은 정렬 옵션에 따라 정렬 위치를 바꾼다.
                        switch( ((GMTBUTTON*)node->ObjAttribute)->Label.nHorzAlign )
                        {

                            case GMT_LEFT:

                                PosX = ( node->Position.x );

                            break;

                            case GMT_CENTER:

                                PosX = ( ( ( node->Position.x << 1 ) + node->Position.w ) >> 1 ) - ( temp->w >> 1 );

                            break;

                            case GMT_RIGHT:

                                PosX = ( ( node->Position.x ) + node->Position.w ) - ( temp->w );

                            break;

                        }

                        switch( ((GMTBUTTON*)node->ObjAttribute)->Label.nVertAlign )
                        {

                            case GMT_TOP:

                                PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->Position.y ) ) ) ) );

                            break;

                            case GMT_CENTER:

                                PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->Position.y << 1 ) + node->Position.h ) >> 1 ) ) + ( temp->h >> 1 ) );

                            break;

                            case GMT_BOTTOM:

                                PosY = ( GMTSystem->GetScreenHeight() - ( ( ( ( node->Position.y ) + node->Position.h ) ) ) + ( temp->h ) );

                            break;

                        }

                        if( !glWindowPos2iARB )
                            glRasterPos2i( PosX, GMTSystem->GetScreenHeight() - PosY );
                        else
                            glWindowPos2iARB( PosX , PosY );

                        glDrawPixels(temp->w, temp->h, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, temp->pixels);
                        glDisable(GL_BLEND);

                    }

                }

            break;

            case GMT_OBJ_DISPLAY:

                // 색상은 현재 오브젝트의 상태 값에 의존하여 처리 합니다.
                // 만약 이미지가 있다면, 이미지로 화면을 그립니다. 이미지는 텍스쳐로 처리 합니다.
                // 알파 블렌딩을 사용하는지 확인하고, 사용 한다면 알파 블렌딩으로 드로우 합니다.
                if( node->ImgDraw )
                {

                    glBindTexture(GL_TEXTURE_2D, ((GMTDISPLAY*)node->ObjAttribute)->BgID);
                    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
                    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
                    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

                    glEnable(GL_TEXTURE_2D);
                    glEnable(GL_ALPHA_TEST);

                    glAlphaFunc(GL_GREATER, 0.9f);

                    if( node->AlphaDraw )
                    {

                        // BlendColor Extension 으로 사용
                        glEnable(GL_BLEND);

                        if( !glBlendColorEXT )
                            ;
                        else
                            glBlendColorEXT(1.0f, 1.0f, 1.0f, ( ((GMTDISPLAY*)node->ObjAttribute)->AlphaValue ? ((GLfloat)((GMTDISPLAY*)node->ObjAttribute)->AlphaValue / 255.0f) : 0.0f ) );

                        glBlendFunc( GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA );

                    }

                    if( node->IsNewUV )
                    {

                        glBegin(GL_QUADS);

                            glTexCoord2f(node->uValue, 0.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                            glTexCoord2f(node->uValue, node->vValue);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                            glTexCoord2f(0.0f, node->vValue);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                            glTexCoord2f(0.0f, 0.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                        glEnd();

                    }
                    else
                    {

                        glBegin(GL_QUADS);

                            glTexCoord2f(1.0f, 0.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                            glTexCoord2f(1.0f, 1.0f);glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                            glTexCoord2f(0.0f, 1.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                            glTexCoord2f(0.0f, 0.0f);glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                        glEnd();

                    }

                    glDisable(GL_TEXTURE_2D);
                    glDisable(GL_ALPHA_TEST);

                    if( node->AlphaDraw )
                        glDisable(GL_BLEND);

                }
                else
                {

                    tempColor = ((GMTDISPLAY*)node->ObjAttribute)->BgColor;

                    if( node->AlphaDraw )
                    {

                        // glBlendColor는 Extension이라서 일단 제외
                        glEnable(GL_BLEND);

                        glColor4f( ( (GLfloat)tempColor.r / 255.0f ), ( (GLfloat)tempColor.g / 255.0f ), ( (GLfloat)tempColor.b / 255.0f ), ( ((GMTDISPLAY*)node->ObjAttribute)->AlphaValue ? ((GLfloat)((GMTDISPLAY*)node->ObjAttribute)->AlphaValue / 255.0f) : 0.0f ) );
                        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

                    }
                    else
                        glColor3f( ( (GLfloat)tempColor.r / 255.0f ), ( (GLfloat)tempColor.g / 255.0f ), ( (GLfloat)tempColor.b / 255.0f ) );

                    glRectf(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y, node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - (node->Position.y + node->Position.h));

                    if( node->AlphaDraw )
                        glDisable(GL_BLEND);

                }

                if( !((GMTDISPLAY*)node->ObjAttribute)->LineWidth )
                {
                }
                else
                {

                    glLineWidth( ((GMTDISPLAY*)node->ObjAttribute)->LineWidth );
                    glColor3f(0.0f, 0.0f, 0.0f);
                    glPolygonMode(GL_FRONT, GL_LINE);

                    glBegin(GL_QUADS);

                        glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - node->Position.y);
                        glVertex2i(node->Position.x, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h ) );
                        glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - ( node->Position.y + node->Position.h) );
                        glVertex2i(node->Position.x + node->Position.w, GMTSystem->GetScreenHeight() - node->Position.y);

                    glEnd();

                    glPolygonMode(GL_FRONT, GL_FILL);

                    glLineWidth(1.0f);

                }

            break;

        }

        if( node->child )
        {

            node = node->child;

        }
        else
        {

            if( node->next == node->parent )
                node = node->parent->next;
            else
                node = node->next;

        }

    }

}

// 마우스와 오브젝트의 처리
void GMTOBJECTSYSTEM::ProcMotionProcess(unsigned short int x, unsigned short int y)
{

    // 사용자 정의 이벤트 설정
    SDL_Event tempEvent;

    tempEvent.type = GMT_MESSAGE;

    // Proc 링크드 리스트를 돌며 마우스와의 위치를 파악한다.
    for(GMTOBJECT *node = ProcList.GetRootData() ; node != ProcList.GetTailData() ; )
    {

        if( node->parent )
        {

            GMT_Rect childRect;

            childRect.x = node->Position.x + node->parent->Position.x;
            childRect.y = node->Position.y + node->parent->Position.y;
            childRect.w = node->Position.w;
            childRect.h = node->Position.h;

            if( GMT_CheckMouseRect( x, y, childRect ) )
            {

                // 오브젝트의 옵션 값을 조정 한다. 단, 클릭 상태가 아닐때.
                if( node->State != GMT_OBJ_CLICK )
                {

                    tempEvent.user.code = GMT_OBJ_OVER;
                    tempEvent.user.data1 = &node->ObjectType;
                    tempEvent.user.data2 = node;
                    node->State = GMT_OBJ_OVER;

                    nProcID = node->nID;

                    SDL_PushEvent(&tempEvent);

                    // 만약 OnOver 이벤트가 있다면 OnOver 이벤트를 수행한다.
                    if( node->IsModule )
                    {

                        if( node->EventList.OnOver )
                            node->EventList.OnOver();

                    }

                }

            }
            else
            {

                if( node->State == GMT_OBJ_OVER )
                {

                    if( node->IsModule )
                    {

                        if( node->EventList.OnOut )
                            node->EventList.OnOut();

                    }

                    nProcID = 0;

                }

                node->State = GMT_OBJ_UNDER;

            }

        }
        else
        {

            if( GMT_CheckMouseRect( x, y, node->Position ) )
            {

                // 오브젝트의 옵션 값을 조정 한다. 단, 클릭 상태가 아닐때.
                if( node->State != GMT_OBJ_CLICK )
                {

                    tempEvent.user.code = GMT_OBJ_OVER;
                    tempEvent.user.data1 = &node->ObjectType;
                    tempEvent.user.data2 = node;
                    node->State = GMT_OBJ_OVER;

                    nProcID = node->nID;

                    SDL_PushEvent(&tempEvent);

                    if( node->IsModule )
                    {

                        if( node->EventList.OnOver )
                            node->EventList.OnOver();

                    }

                }

            }
            else
            {

                if( node->State == GMT_OBJ_OVER )
                {

                    if( node->IsModule )
                    {

                        if( node->EventList.OnOut )
                            node->EventList.OnOut();

                    }

                    nProcID = 0;

                }

                node->State = GMT_OBJ_UNDER;

            }

        }

        if( node->child )
        {

            node = node->child;

        }
        else
        {

            if( node->next == node->parent )
                node = node->parent->next;
            else
                node = node->next;

        }

    }

}

// 마우스와 클릭 처리
void GMTOBJECTSYSTEM::ProcClickProcess(unsigned short int x, unsigned short int y, unsigned char btn, unsigned char state)
{

    SDL_Event tempEvent;
    tempEvent.type = GMT_MESSAGE;

    // Proc 링크드 리스트를 돌며 마우스와의 위치를 파악한다.
    for(GMTOBJECT *node = ProcList.GetRootData() ; node != ProcList.GetTailData() ; )
    {

        if( node->parent )
        {

            GMT_Rect childRect;

            childRect.x = node->Position.x + node->parent->Position.x;
            childRect.y = node->Position.y + node->parent->Position.y;
            childRect.w = node->Position.w;
            childRect.h = node->Position.h;

            if( GMT_CheckMouseRect( x, y, childRect ) )
            {

                if( state == 5 )
                {

                    tempEvent.user.code = GMT_OBJ_CLICK;
                    tempEvent.user.data1 = &node->ObjectType;
                    tempEvent.user.data2 = node;

                    // 오브젝트의 옵션 값을 조정 한다.
                    node->State = GMT_OBJ_CLICK;

                    nProcID = node->nID;

                    SDL_PushEvent(&tempEvent);

                    if( node->IsModule )
                    {

                        if( node->EventList.OnClick )
                            node->EventList.OnClick();

                    }

                }
                else
                {

                    tempEvent.user.code = GMT_OBJ_OVER;
                    tempEvent.user.data1 = &node->ObjectType;
                    tempEvent.user.data2 = node;

                    // 오브젝트의 옵션 값을 조정 한다.
                    node->State = GMT_OBJ_OVER;

                    nProcID = node->nID;

                    SDL_PushEvent(&tempEvent);

                    if( node->IsModule )
                    {

                        if( node->EventList.OnOver )
                            node->EventList.OnOver();

                    }

                }

            }

        }
        else
        {

            if( GMT_CheckMouseRect( x, y, node->Position ) )
            {

                if( state == 5 )
                {

                    tempEvent.user.code = GMT_OBJ_CLICK;
                    tempEvent.user.data1 = &node->ObjectType;
                    tempEvent.user.data2 = node;

                    // 오브젝트의 옵션 값을 조정 한다.
                    node->State = GMT_OBJ_CLICK;

                    nProcID = node->nID;

                    SDL_PushEvent(&tempEvent);

                    if( node->IsModule )
                    {

                        if( node->EventList.OnClick )
                            node->EventList.OnClick();

                    }

                }
                else
                {

                    tempEvent.user.code = GMT_OBJ_OVER;
                    tempEvent.user.data1 = &node->ObjectType;
                    tempEvent.user.data2 = node;

                    // 오브젝트의 옵션 값을 조정 한다.
                    node->State = GMT_OBJ_OVER;

                    nProcID = node->nID;

                    SDL_PushEvent(&tempEvent);

                    if( node->IsModule )
                    {

                        if( node->EventList.OnOver )
                            node->EventList.OnOver();

                    }

                }

            }

        }

        if( node->child )
        {

            node = node->child;

        }
        else
        {

            if( node->next == node->parent )
                node = node->parent->next;
            else
                node = node->next;

        }

    }

}

//==========================================================================================
//
// GMT 관련 함수 들
//
//==========================================================================================

// GMT로 이벤트를 구해오는 루틴
int GMT_GetEvent(SDL_Event *event)
{

    while( SDL_PollEvent(event) )
    {

        switch( event->type )
        {

            case SDL_VIDEORESIZE:

                GMTSystem->ResizeWindow( event->resize.w, event->resize.h );

            break;

            case SDL_MOUSEMOTION:

                GMTSystem->CheckObject( event->motion.x, event->motion.y );

            break;

            case SDL_MOUSEBUTTONDOWN:
            case SDL_MOUSEBUTTONUP:

                GMTSystem->CheckClick( event->button.x, event->button.y, event->button.button, event->button.type );

            break;

        }

        return 1;

    }

    return 0;

}

// GMT로 화면을 지우기 위한 색상을 설정 하는 루틴
void GMT_ClearColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a)
{

    GLfloat Red, Green, Blue, Alpha;

    r ? Red = (GLfloat)( r / 255 ) : Red = 0.0f;
    g ? Green = (GLfloat)( g / 255 ) : Green = 0.0f;
    b ? Blue = (GLfloat)( b / 255 ) : Blue = 0.0f;
    a ? Alpha = (GLfloat)( a / 255 ) : Alpha = 0.0f;

    glClearColor( r, g, b, a );

}

// GMT 로 화면을 지우는 루틴
void GMT_ClearScreen()
{

    GMTSystem->ClearScreen();

}

// GMT 로 화면을 재 갱신 하는 루틴
void GMT_Flip(void)
{

    GMTSystem->Flip();

}
