//===========================================================
//
//  Game Maker Tools - GMT_request.cpp
//
//  - gmt_request.cpp 파일은 질의 응답 관련 처리 함수가 모여 있습니다.
//
//===========================================================

#include "./../header/gmt.h"

// 다른 곳에서 선언된 GMTSystem 클레스.
extern GMTSYSTEM *GMTSystem;

//==========================================================================================
//
// GMTSYSTEM 클레스 부분
//
//==========================================================================================

// 가로, 세로 크기를 구하는 루틴
unsigned short int GMTSYSTEM::GetScreenWidth(void)
{

    // 가로 크기를 반환 합니다.
    return mainscreen->w;

}

unsigned short int GMTSYSTEM::GetScreenHeight(void)
{

    // 세로 크기를 반환 합니다.
    return mainscreen->h;

}

//==========================================================================================
//
// GMTOBJECTSYSTEM 클레스 부분
//
//==========================================================================================

// 링크드 리스트를 순회하며 해당 아이디를 찾습니다.
GMTOBJECT* GMTOBJECTSYSTEM::GetObject(unsigned int nID)
{

    // 루틴이 조금 변경 되었습니다. 링크드 리스트와 자식 노드 까지 모두 순회 하여 찾습니다.
    for( GMTOBJECT *node = ProcList.GetRootData() ; node != ProcList.GetTailData() ; )
    {

        if( node->nID == nID )
        {

            return node;

        }

        if( node->child )
        {

            node = node->child;

        }
        else
        {

            if( node->next == node->parent )
                node = node->parent->next;
            else
                node = node->next;

        }

    }

    return 0;

}

GMTGROUP* GMTOBJECTSYSTEM::GetGroup(unsigned int gID)
{

    GMT_ObjectID GroupID;
    GroupID.nID = 0;
    GroupID.nGroupID = gID | GMT_IS_GROUP;

    // 그룹 리스트를 모두 돌아 ID를 가진 오브젝트를 찾습니다.
    for( GMTGROUP *node = GroupList.GetRootData() ; node != GroupList.GetTailData() ; node = node->next )
    {

        if( node->gID == GroupID.nID )
        {

            return node;

        }

    }

    return 0;

}

GMTOBJECT* GMTOBJECTSYSTEM::PopObject(unsigned int nID)
{

    for( GMTOBJECT *node = ProcList.GetRootData() ; node != ProcList.GetTailData() ; node = node->next )
        if( node->nID == nID )
        {

            // 자신의 데이터를 뺍니다.
            node->prev->next = node->next;
            node->next->prev = node->prev;

            return node;

        }

    return 0;

}

//==========================================================================================
//
// GMT 관련 함수 들
//
//==========================================================================================

// GMT 사각 영역 체크 함수
bool GMT_CheckMouseRect( unsigned short int x, unsigned short int y, GMT_Rect Rect )
{

    if( x > Rect.x + Rect.w ) return 0;
    if( y > Rect.y + Rect.h ) return 0;
    if( x < Rect.x ) return 0;
    if( y < Rect.y ) return 0;

    return 1;

}
