#ifndef __GMT_OBJTYPE__H__
#define __GMT_OBJTYPE__H__

#include "gmt_struct.h"

// 추가된 모듈 라이브러리와 스크립트 라이브러리
#include "gmt_module.h"
#include "gmt_script.h"

// 오브젝트 ID의 하위 2바이트는 일반 오브젝트의 ID로 사용되고, 상위 2바이트는 그룹 ID로 사용 됩니다.
// 따라서 상위 총 15비트(32768)는 그룹의 ID값(0~32768)을 나타내고, 최 상위에 1이 있다면 해당 오브젝트는 그룹에 속해 있음을 뜻합니다.
#define GMT_MAX_OBJECT  65535
#define GMT_MAX_GROUP   32768

// Group 인가 아닌가는 오브젝트 ID의 최 상위 비트가 1인지 아닌지로 판별됩니다.
#define GMT_IS_GROUP32      0x80000000
#define GMT_IS_GROUP16      0x8000
#define GMT_IS_GROUP        GMT_IS_GROUP16

// 화면에 보이는가, 보이지 않는가?
#define GMT_VISIBLE            1
#define GMT_UNVISIBLE        0

// 해당 오브젝트를 사용할 수 있는가, 없는가?
#define GMT_ABLE                1
#define GMT_DISABLE           0

// 해당 오브젝트의 정렬 옵션
#define GMT_LEFT                0
#define GMT_CENTER             1
#define GMT_RIGHT               2
#define GMT_MIDDLE            GMT_CENTER
#define GMT_TOP                 GMT_LEFT
#define GMT_BOTTOM            GMT_RIGHT

// 오브젝트 ID를 저장하기 위한 공용체.
union GMT_ObjectID
{

    struct
    {

        unsigned short nObjID;                                    // 하위 2 바이트는 오브젝트 ID 입니다.
        unsigned short nGroupID;                                  // 상위 2바이트는 그룹 ID 입니다. 단, 최상위 1 비트는 그룹에 속해 있는지 식별자로 쓰입니다.

    };

    unsigned int nID;                                               // 오브젝트 ID를 나타냅니다.

};

// 오브젝트의 특성의 고유 값들
enum { GMT_OBJ_LABEL, GMT_OBJ_BUTTON, GMT_OBJ_DISPLAY, GMT_OBJ_LIST, };

// 오브젝트의 상황별 특성 값
enum { GMT_OBJ_UNDER = 0, GMT_OBJ_OVER = 1, GMT_OBJ_CLICK = 2, };

// 정렬 값
enum { GMT_ALIGN_LEFT, GMT_ALIGN_CENTER, GMT_ALIGN_RIGHT, };

// List 옵션 값
enum { GMT_LIST_OPTION, GMT_LIST_CHECK };

// GMT Label 오브젝트 구조체, 여러가지 상황에서 쓰일 내용을 보여줄 수 있도록 정의되어져 있다.
struct GMTLABEL
{

    // 문자열이 어떻게 저장되는지를 저장, 가로 / 세로 저장 되는 패턴을 저장
    unsigned char nHorzAlign;
    unsigned char nVertAlign;

    // 얼만큼 띄울 것인가
    short int offsetx;
    short int offsety;

    // 화면에 출력할 글자의 배경색상
    GMT_Color LabelBGColor;

    // 화면에 출력할 글자의 색상
    GMT_Color LabelColor;

    // 화면에 표시될 문자열
    char Label[64];

};

// GMT Frame 오브젝트 구조체, Frame에서 사용 가능한 모든 내용이 정의되어져 있다.
// 변경된 GMT Display 오브젝트 구조체. 화면 출력 및 컨테이너 역활을 담당한다.
struct GMTDISPLAY
{

    // 자식들의 위치를 조정하는 offsetx, offsety 값
    unsigned short int offsetx, offsety;

    // 디스플레이의 라인 굵기를 설정, 0이면 라인 없음을 의미
    unsigned char LineWidth;

    // 디스플레이의 라인 색을 설정
    GMT_Color LineColor;

    // 디스플레이의 배경 색을 설정
    GMT_Color BgColor;

    // 디스플레이의 알파 값을 설정
    unsigned char AlphaValue;

    // 디스플레이의 배경 이미지를 설정
    GMT_Image BgImg;                                                                        // 공통, 이미지

    // 임시, OpenGL 텍스쳐 ID로 저장
    unsigned int BgID;

    // 임시, OpenGL 텍스쳐 로딩시 사용될 옵션 값
    int BgTexType;

    // 임시, 라벨
    GMTLABEL Label;

};

// GMT Button 오브젝트 구조체, Button에서 사용 가능한 모든 내용이 정의되어져 있다.
struct GMTBUTTON
{

    // 버튼의 라벨을 설정, 0 이면 라벨 없음
    GMTLABEL Label;

    // 마우스와의 관계 상황을 설정, 상황에 따라 다르게 파악된다.
    struct
    {

        // 버튼의 라인 굵기를 설정, 0이면 라인 없음을 의미
        unsigned char LineWidth;

        // 버튼의 라인 색을 설정
        GMT_Color LineColor;

        // 버튼의 배경 색을 설정
        GMT_Color BgColor;

        // 버튼의 알파 값을 설정
        unsigned char AlphaValue;

        // 배경 이미지를 설정
        GMT_Image BgImg;

        // 임시, OpenGL 텍스쳐 ID로 설정
        unsigned int BgID;

        // 임시, OpenGL 텍스쳐 로딩시 사용될 옵션 값
        int BgTexType;

    }ButtonState[3];

};

// GMT List 오브젝트 구조체, 여러가지 선택을 할 수 있도록 구성되어져 있다.
struct GMTLISTITEM
{

    // 화면에 표시될 문자열
    char ItemName[64];

};

struct GMTLIST
{

    unsigned char ListType;                                                               // List 오브젝트의 특성을 결정, 오직 1개의 리스트만 선택 가능한 Option형인지, 아니면 Check 형인지 설정 기본은 Option

    unsigned char nTotalItem;                                                            // 총 아이템의 갯수를 반환 한다. List는 최대 255개의 아이템을 가질 수 있다.

    GMT_Color BGColor;                                                                     // 리스트의 배경 색

    GMTLISTITEM *ListItem;                                                              // 실제 List 데이터가 들어갈 Item 리스트, 들어가는 순서대로 ID값이 저장된다.

    unsigned int nSelectItem[8];                                                          // 선택된 아이템의 ID를 가진다. 총 255개의 값을 저장할 255개의 비트를 의미

};

// GMT Object 구조체, 이 구조체는 오브젝트 들을 관리 감독하기 위해 사용되어 집니다. 외부에서 생성되어져선 안 됩니다.
struct GMTOBJECT
{

    unsigned char ObjectType;                                                            // 오브젝트의 타입. 종류는 위에 열거 되어져 있다.

    unsigned int nID;                                                                       // ID 값의 저장

    // 실제 오브젝트의 데이터 들이 들어갈 곳.
    bool IsVisible;                                                                            // 화면에 출력을 할 것인지를 나타낸다.
    bool IsAble;                                                                               // 사용이 가능한 것인지를 나타낸다.

    // 오브젝트의 상태 값
    unsigned char ImgDraw;                                                               // 이미지로 그릴 것인지 아니면 색상으로 그릴 것인지
    unsigned char AlphaDraw;                                                            // 알파 블렌딩을 사용할 것인지 설정
    unsigned char State;                                                                   // 오브젝트의 상태 값

    GMT_Rect Position;                                                                      // 오브젝트의 위치를 나타내는 값

    void *ObjAttribute;                                                                     // 오브젝트의 속성 값을 저장

    GMTOBJECT *parent;                                                                   // 자신에게 부모가 있다면 이곳에는 부모를 가르키게 됩니다.

    GMTOBJECT *prev;                                                                      // 링크드 리스트용 데이터, 이전 데이터를 가르킴
    GMTOBJECT *next;                                                                      // 링크드 리스트용 데이터, 다음 데이터를 가르킴

    GMTOBJECT *child;                                                                      // 자신에게 자식이 있다면 이곳에는 자식을 가르키게 됩니다.

    // 새로 추가된 저 사양용 UV 좌표 계산 값.
    bool IsNewUV;                                                                           // 새 UV를 쓸 것인가?

    float uValue;                                                                              // 새 UV의 U 값
    float vValue;                                                                              // 새 UV의 V 값

    // 오브젝트 모듈과 오브젝트 스크립트 구문.
    bool IsScript;                                                                             // 스크립트가 있는지 확인 합니다.
    EVENTSCRIPT ScriptList;                                                              // 마찬가지로 각각의 상황에 대한 스크립트를 적재 할 수 있습니다.

    bool IsModule;                                                                           // 적재된 모듈이 있는지 확인 합니다.
    EVENTMODULE EventList;                                                             // 오브젝트는 각각의 상황에 대한 이벤트 모듈들을 적재 할 수 있습니다.

};

// GMT Group 구조체, 오브젝트를 한곳으로 묶기 위해 사용됩니다.
struct GMTGROUP
{

    unsigned int gID;                                                                       // ID 값의 저장

    unsigned int nTotalMember;                                                          // 총 멤버를 저장
    unsigned int *Member;                                                             // 멤버의 ID를 저장해 둡니다.

    GMTGROUP *prev;                                                                      // 링크드 리스트용 데이터, 이전 데이터를 가르킴
    GMTGROUP *next;                                                                      // 링크드 리스트용 데이터, 이전 데이터를 가르킴

};

#endif
